@@ -1,6 +1,6 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 #![doc = include_str!("../README.md")]
 use configparser::ini::Ini;
@@ -58,7 +58,7 @@ fn get(con: &mut redis::Connection) -> RedisResult<HashMap<String, Vec<String>>>
         redis::Cmd::new()
             .arg("SELECT")
             .arg(i.to_string())
-            .query(con)?;
+            .query::<()>(con)?;
         let result = get_all(con)?;
         if result.len() > 1 {
             return Ok(result);
@@ -159,28 +159,31 @@ fn main() {
     let nasl_cli = match env::current_exe() {
         Ok(mut x) => {
             x.pop();
-            x.push("nasl-cli");
+            x.push("scannerctl");
             x
         }
-        Err(x) => panic!("This test program is assuming that nasl-cli is in the same dir: {x}"),
+        Err(x) => panic!("This test program is assuming that scannerctl is in the same dir: {x}"),
     };
     let (ncd, nasl_cli) = run_get(
         &mut kb,
-        &format!("{} feed update", nasl_cli.to_str().unwrap_or_default()),
+        &format!(
+            "{} feed update --vts-only",
+            nasl_cli.to_str().unwrap_or_default()
+        ),
     )
     .expect("results");
     let mut errors = 0;
     if ncd > od {
         errors += print_error(&format!(
-            "openvas ({od:?}) was faster than nasl-cli ({ncd:?})"
+            "openvas ({od:?}) was faster than scannerctl ({ncd:?})"
         ));
     }
 
     let (left, right) = {
-        println!("nasl-cli: {} entries", nasl_cli.len());
+        println!("scannerctl: {} entries", nasl_cli.len());
         println!("openvas: {} entries", openvas.len());
         if nasl_cli.len() > openvas.len() {
-            println!("nasl-cli is left");
+            println!("scannerctl is left");
             (nasl_cli, openvas)
         } else {
             println!("openvas is left");
