@@ -1,13 +1,13 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 #![doc = include_str!("../README.md")]
 #![warn(missing_docs)]
 mod oid;
 pub mod transpile;
 mod update;
-mod verify;
+pub mod verify;
 
 pub use oid::Oid;
 pub use update::feed_version as version;
@@ -19,3 +19,4 @@ pub use verify::FileNameLoader;
 pub use verify::HashSumNameLoader;
 pub use verify::Hasher;
 pub use verify::NaslFileFinder;
+pub use verify::SignatureChecker;
