@@ -1,15 +1,18 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 //! Is a module to get oids within a feed
 
-use std::fs::File;
+use std::{fs::File, io::Read};
 
 use nasl_interpreter::{AsBufReader, Loader};
-use nasl_syntax::{IdentifierType, Statement, TokenCategory};
+use nasl_syntax::{IdentifierType, Statement, StatementKind, TokenCategory};
 
-use crate::{update, verify};
+use crate::{
+    update,
+    verify::{self, HashSumFileItem},
+};
 /// Updates runs nasl plugin with description true and uses given storage to store the descriptive
 /// information
 pub struct Oid<L, V> {
@@ -17,10 +20,11 @@ pub struct Oid<L, V> {
     loader: L,
     verifier: V,
 }
-impl<L, V> Oid<L, V>
+impl<'a, L, V, R> Oid<L, V>
 where
     L: Sync + Send + Loader + AsBufReader<File>,
-    V: Iterator<Item = Result<String, verify::Error>>,
+    V: Iterator<Item = Result<HashSumFileItem<'a, R>, verify::Error>>,
+    R: Read + 'a,
 {
     /// Creates an oid finder. Returns a tuple of (filename, oid).
     ///
@@ -36,10 +40,11 @@ where
     }
 
     fn script_oid(stmt: &Statement) -> Option<String> {
-        match stmt {
-            Statement::Call(t, stms, _) => match t.category() {
+        match stmt.kind() {
+            StatementKind::Call(param) => match stmt.start().category() {
                 TokenCategory::Identifier(IdentifierType::Undefined(s)) => match s as &str {
-                    "script_oid" => stms.first().map(|x| x.to_string()),
+                    // maybe switch from children to patternmatching?
+                    "script_oid" => param.children().first().map(|x| x.to_string()),
                     _ => None,
                 },
                 _ => None,
@@ -52,8 +57,8 @@ where
     fn single(&self, key: String) -> Result<String, update::ErrorKind> {
         let code = self.loader.load(key.as_ref())?;
         for stmt in nasl_syntax::parse(&code) {
-            if let Statement::If(_, _, stmts, _, _) = stmt? {
-                if let Statement::Block(_, x, _) = &*stmts {
+            if let StatementKind::If(_, stmts, _, _) = stmt?.kind() {
+                if let StatementKind::Block(x) = stmts.kind() {
                     for stmt in x {
                         if let Some(oid) = Self::script_oid(stmt) {
                             return Ok(oid);
@@ -66,26 +71,35 @@ where
     }
 }
 
-impl<L, V> Iterator for Oid<L, V>
+impl<'a, L, V, R> Iterator for Oid<L, V>
 where
     L: Sync + Send + Loader + AsBufReader<File>,
-    V: Iterator<Item = Result<String, verify::Error>>,
+    V: Iterator<Item = Result<HashSumFileItem<'a, R>, verify::Error>>,
+    R: Read + 'a,
 {
     type Item = Result<(String, String), update::Error>;
 
     fn next(&mut self) -> Option<Self::Item> {
         match self.verifier.find(|x| {
             if let Ok(x) = x {
-                x.ends_with(".nasl")
+                x.get_filename().ends_with(".nasl")
             } else {
                 true
             }
         }) {
-            Some(Ok(k)) => Some(
-                self.single(k.clone())
-                    .map(|x| (k.clone(), x))
-                    .map_err(|e| update::Error { kind: e, key: k }),
-            ),
+            Some(Ok(k)) => {
+                if let Err(e) = k.verify() {
+                    return Some(Err(e.into()));
+                }
+                Some(
+                    self.single(k.get_filename())
+                        .map(|x| (k.get_filename(), x))
+                        .map_err(|e| update::Error {
+                            kind: e,
+                            key: k.get_filename(),
+                        }),
+                )
+            }
             Some(Err(e)) => Some(Err(e.into())),
             None => None,
         }
