@@ -1,42 +1,51 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
-use nasl_interpreter::{InterpretError, LoadError};
-use nasl_syntax::SyntaxError;
+use nasl_interpreter::InterpretError;
+use nasl_syntax::{LoadError, SyntaxError};
 use storage::StorageError;
+use thiserror::Error;
 
 use crate::verify;
 
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Error)]
 /// Errors within feed handling
 pub enum ErrorKind {
     /// An InterpretError occurred while interpreting
-    InterpretError(InterpretError),
+    #[error("Interpreter error: {0}")]
+    InterpretError(#[from] InterpretError),
     /// NASL script contains an SyntaxError
-    SyntaxError(SyntaxError),
+    #[error("Syntax error: {0}")]
+    SyntaxError(#[from] SyntaxError),
     /// Storage is unable to handle operation
-    StorageError(StorageError),
+    #[error("Storage error: {0}")]
+    StorageError(#[from] StorageError),
     /// Loader is unable to handle operation
-    LoadError(LoadError),
+    #[error("Load error: {0}")]
+    LoadError(#[from] LoadError),
     /// Description if block without exit
+    #[error("Missing exit: {0}")]
     MissingExit(String),
     /// Describes an error while verifying the file
-    VerifyError(verify::Error),
+    #[error("Verify error: {0}")]
+    VerifyError(#[from] verify::Error),
 }
 
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Error)]
+#[error("Error with key '{key}': {kind}")]
 /// ErrorKind and key of error
 pub struct Error {
     /// Used key for the operation
     pub key: String,
-    /// The kind of error occurred
+    /// The kind of the error
+    #[source]
     pub kind: ErrorKind,
 }
 
 impl From<verify::Error> for Error {
     fn from(value: verify::Error) -> Self {
-        let fin = match &value {
+        let key = match &value {
             crate::VerifyError::SumsFileCorrupt(x) => x.sum_file(),
             crate::VerifyError::LoadError(_) => "",
             crate::VerifyError::HashInvalid {
@@ -48,32 +57,8 @@ impl From<verify::Error> for Error {
             crate::VerifyError::MissingKeyring => "",
         };
         Self {
-            key: fin.to_string(),
+            key: key.to_string(),
             kind: ErrorKind::VerifyError(value),
         }
     }
 }
-
-impl From<LoadError> for ErrorKind {
-    fn from(value: LoadError) -> Self {
-        Self::LoadError(value)
-    }
-}
-
-impl From<StorageError> for ErrorKind {
-    fn from(value: StorageError) -> Self {
-        Self::StorageError(value)
-    }
-}
-
-impl From<SyntaxError> for ErrorKind {
-    fn from(value: SyntaxError) -> Self {
-        Self::SyntaxError(value)
-    }
-}
-
-impl From<InterpretError> for ErrorKind {
-    fn from(value: InterpretError) -> Self {
-        Self::InterpretError(value)
-    }
-}
