@@ -1,80 +1,78 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 mod error;
 
 pub use error::Error;
+pub use error::ErrorKind;
 
-use std::{fmt::Display, fs::File, marker::PhantomData};
+use std::{fs::File, io::Read};
 
 use nasl_interpreter::{
-    logger::DefaultLogger, AsBufReader, Context, ContextType, Interpreter, Loader, NaslValue,
-    Register,
+    AsBufReader, CodeInterpreter, Context, ContextType, Interpreter, Loader, NaslValue, Register,
 };
-use storage::{nvt::NVTField, Dispatcher, NoOpRetriever};
+use storage::{item::NVTField, ContextKey, Dispatcher, NoOpRetriever};
 
-use crate::verify::{self, signature_check};
-
-pub use self::error::ErrorKind;
+use crate::verify::{self, HashSumFileItem, SignatureChecker};
 
 /// Updates runs nasl plugin with description true and uses given storage to store the descriptive
 /// information
-pub struct Update<S, L, V, K> {
+pub struct Update<'a, S, L, V> {
     /// Is used to store data
-    dispatcher: S,
+    dispatcher: &'a S,
     /// Is used to load nasl plugins by a relative path
-    loader: L,
+    loader: &'a L,
     /// Initial data, usually set in new.
     initial: Vec<(String, ContextType)>,
     /// How often loader or storage should retry before giving up when a retryable error occurs.
     max_retry: usize,
     verifier: V,
     feed_version_set: bool,
-    phanton: PhantomData<K>,
 }
 
-impl From<verify::Error> for ErrorKind {
-    fn from(value: verify::Error) -> Self {
-        ErrorKind::VerifyError(value)
-    }
-}
 /// Loads the plugin_feed_info and returns the feed version
-pub fn feed_version<K: Default + AsRef<str>>(
-    loader: &dyn Loader,
-    dispatcher: &dyn Dispatcher<K>,
-) -> Result<String, ErrorKind> {
+pub fn feed_version(loader: &dyn Loader, dispatcher: &dyn Dispatcher) -> Result<String, ErrorKind> {
     let feed_info_key = "plugin_feed_info.inc";
     let code = loader.load(feed_info_key)?;
-    let mut register = Register::default();
-    let logger = DefaultLogger::default();
-    let k: K = Default::default();
+    let register = Register::default();
+    let k = ContextKey::default();
     let fr = NoOpRetriever::default();
     let target = String::default();
     // TODO add parameter to struct
     let functions = nasl_interpreter::nasl_std_functions();
-    let context = Context::new(&k, &target, dispatcher, &fr, loader, &logger, &functions);
-    let mut interpreter = Interpreter::new(&mut register, &context);
+    let context = Context::new(k, target, dispatcher, &fr, loader, &functions);
+    let mut interpreter = Interpreter::new(register, &context);
     for stmt in nasl_syntax::parse(&code) {
         match stmt {
-            Ok(stmt) => interpreter.retry_resolve(&stmt, 3)?,
+            Ok(stmt) => interpreter.retry_resolve_next(&stmt, 3)?,
             Err(e) => return Err(e.into()),
         };
     }
 
-    let feed_version = register
+    let feed_version = interpreter
+        .register()
         .named("PLUGIN_SET")
         .map(|x| x.to_string())
         .unwrap_or_else(|| "0".to_owned());
     Ok(feed_version)
 }
 
-impl<S, L, V, K> Update<S, L, V, K>
+impl<'a, R, S, L, V> SignatureChecker for Update<'a, S, L, V>
 where
-    S: Sync + Send + Dispatcher<K>,
-    K: AsRef<str> + Display + Default + From<String>,
+    S: Sync + Send + Dispatcher,
     L: Sync + Send + Loader + AsBufReader<File>,
-    V: Iterator<Item = Result<String, verify::Error>>,
+    V: Iterator<Item = Result<HashSumFileItem<'a, R>, verify::Error>>,
+    R: Read + 'a,
+{
+}
+
+impl<'a, S, L, V, R> Update<'a, S, L, V>
+where
+    S: Sync + Send + Dispatcher,
+    L: Sync + Send + Loader + AsBufReader<File>,
+    V: Iterator<Item = Result<HashSumFileItem<'a, R>, verify::Error>>,
+    R: Read + 'a,
 {
     /// Creates an updater. This updater is implemented as a iterator.
     ///
@@ -86,8 +84,8 @@ where
     pub fn init(
         openvas_version: &str,
         max_retry: usize,
-        loader: L,
-        storage: S,
+        loader: &'a L,
+        storage: &'a S,
         verifier: V,
     ) -> Self {
         let initial = vec![
@@ -101,13 +99,23 @@ where
             dispatcher: storage,
             verifier,
             feed_version_set: false,
-            phanton: PhantomData,
         }
     }
 
     /// Loads the plugin_feed_info and returns the feed version
     pub fn feed_version(&self) -> Result<String, ErrorKind> {
-        feed_version(&self.loader, &self.dispatcher)
+        feed_version(self.loader, self.dispatcher)
+    }
+
+    /// Check if the current feed is outdated.
+    pub fn feed_is_outdated(&self, current_version: String) -> Result<bool, ErrorKind> {
+        // the version in file
+        let v = self.feed_version()?;
+        if !current_version.is_empty() {
+            return Ok(v.as_str() != current_version.as_str());
+        };
+
+        Ok(true)
     }
 
     /// plugin_feed_info must be handled differently.
@@ -128,28 +136,25 @@ where
     }
 
     /// Runs a single plugin in description mode.
-    fn single(&self, key: &K) -> Result<i64, ErrorKind> {
-        let code = self.loader.load(key.as_ref())?;
+    fn single(&self, key: &ContextKey) -> Result<i64, ErrorKind> {
+        let code = self.loader.load(&key.value())?;
 
-        let mut register = Register::root_initial(&self.initial);
-        let logger = DefaultLogger::default();
+        let register = Register::root_initial(&self.initial);
         let fr = NoOpRetriever::default();
         let target = String::default();
-        // TODO add parameter to struct
         let functions = nasl_interpreter::nasl_std_functions();
 
         let context = Context::new(
-            key,
-            &target,
-            &self.dispatcher,
+            key.clone(),
+            target,
+            self.dispatcher,
             &fr,
-            &self.loader,
-            &logger,
+            self.loader,
             &functions,
         );
-        let mut interpreter = Interpreter::new(&mut register, &context);
-        for stmt in nasl_syntax::parse(&code) {
-            match interpreter.retry_resolve(&stmt?, self.max_retry) {
+        let interpreter = CodeInterpreter::new(&code, register, &context);
+        for stmt in interpreter {
+            match stmt {
                 Ok(NaslValue::Exit(i)) => {
                     self.dispatcher.on_exit()?;
                     return Ok(i);
@@ -158,39 +163,50 @@ where
                 Err(e) => return Err(e.into()),
             }
         }
-        Err(ErrorKind::MissingExit(key.as_ref().into()))
+        Err(ErrorKind::MissingExit(key.value()))
     }
     /// Perform a signature check of the sha256sums file
     pub fn verify_signature(&self) -> Result<(), verify::Error> {
+        //self::SignatureChecker::signature_check(&path)
         let path = self.loader.root_path().unwrap();
-        signature_check(&path)
+        crate::verify::check_signature(&path)
     }
 }
 
-impl<S, L, V, K> Iterator for Update<S, L, V, K>
+impl<'a, S, L, V, R> Iterator for Update<'a, S, L, V>
 where
-    S: Sync + Send + Dispatcher<K>,
+    S: Sync + Send + Dispatcher,
     L: Sync + Send + Loader + AsBufReader<File>,
-    V: Iterator<Item = Result<String, verify::Error>>,
-    K: AsRef<str> + Display + Default + From<String>,
+    V: Iterator<Item = Result<HashSumFileItem<'a, R>, verify::Error>>,
+    R: Read + 'a,
 {
     type Item = Result<String, Error>;
 
     fn next(&mut self) -> Option<Self::Item> {
         match self.verifier.find(|x| {
             if let Ok(x) = x {
-                x.ends_with(".nasl")
+                x.get_filename().ends_with(".nasl")
             } else {
                 true
             }
         }) {
             Some(Ok(k)) => {
-                let k: K = k.into();
+                if let Err(e) = k.verify() {
+                    return Some(Err(e.into()));
+                }
+
+                let mut filename = k.get_filename();
+                if filename.starts_with("./") {
+                    // sha256sums may start with ./ so we have to remove those as dependencies
+                    // within nasl scripts usually don't entail them.
+                    filename = filename[2..].to_string();
+                }
+                let k = ContextKey::FileName(filename.clone());
                 self.single(&k)
-                    .map(|_| k.as_ref().into())
+                    .map(|_| k.value())
                     .map_err(|kind| Error {
                         kind,
-                        key: k.to_string(),
+                        key: k.value(),
                     })
                     .into()
             }
