@@ -1,9 +1,11 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 use std::collections::HashMap;
 
+use crate::Specifier;
+
 use super::port::Protocol;
 
 /// Scan result
@@ -57,9 +59,42 @@ pub struct Result {
 
     #[cfg_attr(
         feature = "serde_support",
-        serde(skip_serializing_if = "HashMap::is_empty", default)
+        serde(skip_serializing_if = "Option::is_none", default)
     )]
-    pub details: HashMap<String, String>,
+    /// Details are only set on status and can be ignored
+    pub detail: Option<Detail>,
+}
+
+/// Host Details information
+#[derive(Debug, Clone, PartialEq, Eq, Default)]
+#[cfg_attr(
+    feature = "serde_support",
+    derive(serde::Serialize, serde::Deserialize)
+)]
+
+pub struct Detail {
+    /// Descriptive name of a Host Detail
+    pub name: String,
+    /// Detected detail information
+    pub value: String,
+    /// Information about the source of the information
+    pub source: Source,
+}
+
+/// Host details source information
+#[derive(Debug, Clone, PartialEq, Eq, Default)]
+#[cfg_attr(
+    feature = "serde_support",
+    derive(serde::Serialize, serde::Deserialize)
+)]
+pub struct Source {
+    #[serde(rename = "type")]
+    /// type of the source
+    pub s_type: String,
+    /// source reference, e.g. an OID in case of a nvt type
+    pub name: String,
+    /// description about the source
+    pub description: String,
 }
 
 // is used for enumerate handling
@@ -95,3 +130,27 @@ pub enum ResultType {
     /// Detail information about the host
     HostDetail,
 }
+
+/// Notus Results are a Map from OIDs to vulnerable Packages
+pub type NotusResults = HashMap<String, Vec<VulnerablePackage>>;
+
+#[cfg_attr(feature = "serde_support", derive(serde::Serialize))]
+#[derive(Debug)]
+pub struct VulnerablePackage {
+    pub name: String,
+    pub installed_version: String,
+    pub fixed_version: FixedVersion,
+}
+
+#[cfg_attr(feature = "serde_support", derive(serde::Serialize), serde(untagged))]
+#[derive(Debug)]
+pub enum FixedVersion {
+    Single {
+        version: String,
+        specifier: Specifier,
+    },
+    Range {
+        start: String,
+        end: String,
+    },
+}
