@@ -1,8 +1,8 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
-use super::{scanner_preference::ScannerPreference, target::Target, vt::VT};
+use super::{scanner_preference::ScanPreference, target::Target, vt::VT};
 
 /// Struct for creating and getting a scan
 #[derive(Default, Debug, Clone, PartialEq, Eq)]
@@ -11,16 +11,18 @@ use super::{scanner_preference::ScannerPreference, target::Target, vt::VT};
     derive(serde::Serialize, serde::Deserialize),
     serde(deny_unknown_fields)
 )]
-#[cfg_attr(feature = "bincode_support", derive(bincode::Encode, bincode::Decode))]
 pub struct Scan {
     #[cfg_attr(feature = "serde_support", serde(default))]
     /// Unique ID of a scan
-    pub scan_id: Option<String>,
+    pub scan_id: String,
     /// Information about the target to scan
     pub target: Target,
-    #[cfg_attr(feature = "serde_support", serde(default))]
-    /// Configuration options for the scanner
-    pub scanner_preferences: Vec<ScannerPreference>,
+    #[cfg_attr(
+        feature = "serde_support",
+        serde(default, alias = "scanner_preferences")
+    )]
+    /// Configuration options for a scan
+    pub scan_preferences: Vec<ScanPreference>,
     /// List of VTs to execute for the target
     pub vts: Vec<VT>,
 }
