@@ -1,6 +1,6 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 use nasl_builtin_utils::error::FunctionErrorKind;
 use nasl_builtin_utils::{Context, NaslFunction};
@@ -14,6 +14,8 @@ pub mod aes_cmac;
 pub mod aes_ctr;
 pub mod aes_gcm;
 pub mod aes_gmac;
+pub mod des;
+pub mod hash;
 pub mod hmac;
 
 enum Crypt {
@@ -21,13 +23,7 @@ enum Crypt {
     Decrypt,
 }
 
-/// Decodes given string as hex and returns the result as a byte array
-// TODO only used in tests, move tests to its own module and define there
-
-pub(crate) fn lookup<K>(function_name: &str) -> Option<NaslFunction<K>>
-where
-    K: AsRef<str>,
-{
+pub(crate) fn lookup(function_name: &str) -> Option<NaslFunction> {
     aes_ccm::lookup(function_name)
         .or_else(|| hmac::lookup(function_name))
         .or_else(|| aes_cbc::lookup(function_name))
@@ -35,22 +31,24 @@ where
         .or_else(|| aes_gcm::lookup(function_name))
         .or_else(|| aes_cmac::lookup(function_name))
         .or_else(|| aes_gmac::lookup(function_name))
+        .or_else(|| hash::lookup(function_name))
+        .or_else(|| des::lookup(function_name))
 }
 
 pub struct Cryptographic;
 
-impl<K: AsRef<str>> nasl_builtin_utils::NaslFunctionExecuter<K> for Cryptographic {
+impl nasl_builtin_utils::NaslFunctionExecuter for Cryptographic {
     fn nasl_fn_execute(
         &self,
         name: &str,
         register: &Register,
-        context: &Context<K>,
+        context: &Context,
     ) -> Option<nasl_builtin_utils::NaslResult> {
         lookup(name).map(|x| x(register, context))
     }
 
     fn nasl_fn_defined(&self, name: &str) -> bool {
-        lookup::<K>(name).is_some()
+        lookup(name).is_some()
     }
 }
 /// Get named argument of Type Data or String from the register with appropriate error handling.
@@ -64,8 +62,12 @@ fn get_required_named_data<'a>(
     match register.named(key) {
         Some(ContextType::Value(NaslValue::Data(x))) => Ok(x.as_slice()),
         Some(ContextType::Value(NaslValue::String(x))) => Ok(x.as_bytes()),
-        Some(x) => Err((key, "a String or Data Value", format!("{:?}", x).as_str()).into()),
-        _ => Err((key).into()),
+        Some(x) => Err(FunctionErrorKind::wrong_argument(
+            key,
+            "a String or Data Value",
+            format!("{:?}", x).as_str(),
+        )),
+        _ => Err(FunctionErrorKind::missing_argument(key)),
     }
 }
 
@@ -79,7 +81,11 @@ fn get_optional_named_number(
 ) -> Result<Option<i64>, FunctionErrorKind> {
     match register.named(key) {
         Some(ContextType::Value(NaslValue::Number(x))) => Ok(Some(*x)),
-        Some(x) => Err((key, "a Number Value", format!("{:?}", x).as_str()).into()),
+        Some(x) => Err(FunctionErrorKind::wrong_argument(
+            key,
+            "a Number Value",
+            format!("{:?}", x).as_str(),
+        )),
         _ => Ok(None),
     }
 }
