@@ -1,8 +1,7 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
-// TODO replace if (verbose) calls to debug calls
 // TODO make error handling less redundant
 // TODO clean up and maybe split as 2000 lines is a bit much
 //! Defines NASL ssh and sftp functions
@@ -12,7 +11,6 @@ mod sessions;
 use core::str;
 use libssh_rs::{AuthMethods, AuthStatus, Channel, LogLevel, Session, SshKey, SshOption};
 use nasl_builtin_utils::{Context, ContextType, FunctionErrorKind, Register};
-use nasl_syntax::logger::NaslLogger;
 use nasl_syntax::NaslValue;
 use sessions::SshSession;
 use std::io::Write;
@@ -24,8 +22,9 @@ use std::{
     sync::{Arc, Mutex},
     time::Duration,
 };
+use tracing::{debug, info};
 
-type NaslSSHFunction<K> = fn(&Ssh, &Register, &Context<K>) -> Result<NaslValue, FunctionErrorKind>;
+type NaslSSHFunction = fn(&Ssh, &Register, &Context) -> Result<NaslValue, FunctionErrorKind>;
 
 fn read_ssh_blocking(channel: &Channel, timeout: Duration, response: &mut String) -> i32 {
     let mut buf: [u8; 4096] = [0; 4096];
@@ -208,11 +207,10 @@ fn set_opt_user(
 fn get_authmethods(
     session: &mut SshSession,
     session_id: i32,
-    logger: &dyn NaslLogger,
 ) -> Result<AuthMethods, FunctionErrorKind> {
     match session.session.userauth_none(None) {
         Ok(libssh_rs::AuthStatus::Success) => {
-            logger.info(&"SSH authentication succeeded using the none method - should not happen; very old server?");
+            info!("SSH authentication succeeded using the none method - should not happen; very old server?");
             session.authmethods = AuthMethods::NONE;
             session.authmethods_valid = true;
             Ok(AuthMethods::NONE)
@@ -224,10 +222,7 @@ fn get_authmethods(
                 Ok(list)
             }
             Err(_) => {
-                logger.debug(
-                    &"SSH server did not return a list of authentication methods - trying all"
-                        .to_string(),
-                );
+                debug!("SSH server did not return a list of authentication methods - trying all");
                 let methods = AuthMethods::HOST_BASED
                     | AuthMethods::INTERACTIVE
                     | AuthMethods::NONE
@@ -287,11 +282,9 @@ fn channel_read(
 fn exec_ssh_cmd(
     session: &SshSession,
     cmd: &str,
-    verbose: bool,
     compat_mode: bool,
     to_stdout: i32,
     to_stderr: i32,
-    logger: &dyn NaslLogger,
 ) -> Result<(String, String), FunctionErrorKind> {
     let channel = match session.session.new_channel() {
         Ok(c) => c,
@@ -322,12 +315,10 @@ fn exec_ssh_cmd(
     match channel.request_pty("xterm", 80, 24) {
         Ok(_) => (),
         Err(e) => {
-            if verbose {
-                logger.info(&format!(
-                    "Channel failed to request pty for session ID {}: {}",
-                    session.session_id, e
-                ));
-            }
+            debug!(
+                session_id=session.session_id, error=%e,
+                "Channel failed to request pty for session ID",
+            );
         }
     }
 
@@ -412,13 +403,13 @@ impl Ssh {
     /// - scciphers SSH server-to-client ciphers.
     ///
     /// - timeout Set a timeout for the connection in seconds. Defaults to 10
-    /// seconds (defined by libssh internally) if not given.
+    ///   seconds (defined by libssh internally) if not given.
     ///
     /// nasl return An integer to identify the ssh session. Zero on error.
-    fn nasl_ssh_connect<K>(
+    fn nasl_ssh_connect(
         &self,
         register: &Register,
-        ctx: &Context<K>,
+        ctx: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let sock: i64 = register
             .named("socket")
@@ -480,10 +471,10 @@ impl Ssh {
             .unwrap_or(0);
 
         let log_level = match verbose {
-            verbose if verbose <= 0 => LogLevel::NoLogging,
-            verbose if verbose <= 1 => LogLevel::Warning,
-            verbose if verbose <= 2 => LogLevel::Protocol,
-            verbose if verbose <= 3 => LogLevel::Packet,
+            0 => LogLevel::NoLogging,
+            1 => LogLevel::Warning,
+            2 => LogLevel::Protocol,
+            3 => LogLevel::Packet,
             _ => LogLevel::Functions,
         };
         let option = SshOption::LogLevel(log_level);
@@ -561,14 +552,12 @@ impl Ssh {
             let my_sock = UdpSocket::bind("127.0.0.1:0").unwrap();
             let option = SshOption::Socket(my_sock.as_raw_fd());
 
-            if verbose > 0 {
-                ctx.logger().info(&format!(
-                    "Setting SSH fd for '{}' to {} (NASL sock={}",
-                    ip_str,
-                    my_sock.as_raw_fd(),
-                    sock
-                ));
-            }
+            debug!(
+                ip_str = ip_str,
+                sock_fd = my_sock.as_raw_fd(),
+                nasl_sock = sock,
+                "Setting SSH fd for socket",
+            );
 
             if let Err(err) = session.set_option(option) {
                 return Err(FunctionErrorKind::Dirty(
@@ -580,12 +569,10 @@ impl Ssh {
             forced_sock = sock; // TODO: check and fix everything related to open socket
         }
 
-        if verbose > 0 {
-            ctx.logger().info(&format!(
-                "Connecting to SSH server '{}' (port {}, sock {})",
-                ip_str, port, sock
-            ));
-        }
+        debug!(
+            "Connecting to SSH server '{}' (port {}, sock {})",
+            ip_str, port, sock
+        );
 
         match session.connect() {
             Ok(_) => {
@@ -599,7 +586,6 @@ impl Ssh {
                     authmethods: AuthMethods::NONE,
                     authmethods_valid: false,
                     user_set: false,
-                    verbose,
                     channel: None,
                 };
 
@@ -627,10 +613,10 @@ impl Ssh {
     ///
     /// nasl params
     /// - An SSH session id.  A value of 0 is allowed and acts as a NOP.
-    fn nasl_ssh_disconnect<K>(
+    fn nasl_ssh_disconnect(
         &self,
         register: &Register,
-        _ctx: &Context<K>,
+        _ctx: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -671,10 +657,10 @@ impl Ssh {
     ///
     /// return An integer with the corresponding ssh session id or 0 if
     ///          no session id is known for the given socket.
-    fn nasl_ssh_session_id_from_sock<K>(
+    fn nasl_ssh_session_id_from_sock(
         &self,
         register: &Register,
-        _ctx: &Context<K>,
+        _ctx: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -701,10 +687,10 @@ impl Ssh {
     /// - An SSH session id.
     ///  
     /// return An integer representing the socket or -1 on error.
-    fn nasl_ssh_get_sock<K>(
+    fn nasl_ssh_get_sock(
         &self,
         register: &Register,
-        _ctx: &Context<K>,
+        _ctx: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -740,10 +726,10 @@ impl Ssh {
     ///  
     /// nasl named params
     /// - login: A string with the login name (optional).
-    fn nasl_ssh_set_login<K>(
+    fn nasl_ssh_set_login(
         &self,
         register: &Register,
-        _ctx: &Context<K>,
+        _ctx: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -764,7 +750,7 @@ impl Ssh {
 
         let login = match register.named("login") {
             Some(ContextType::Value(NaslValue::String(x))) => Some(x.to_owned()),
-            _ => return Err(FunctionErrorKind::from("login")),
+            _ => return Err(FunctionErrorKind::missing_argument("login")),
         };
 
         let mut sessions = lock_sessions(&self.sessions)?;
@@ -829,10 +815,10 @@ impl Ssh {
     /// - passphrase: A string with the passphrase used to unprotect privatekey.
     ///  
     /// return An integer as status value; 0 indicates success.
-    fn nasl_ssh_userauth<K>(
+    fn nasl_ssh_userauth(
         &self,
         register: &Register,
-        ctx: &Context<K>,
+        _: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -851,46 +837,21 @@ impl Ssh {
             }
         };
 
-        // Login is optional. It must be later check if the login was
+        let get_named_val = |name| match register.named(name) {
+            Some(ContextType::Value(NaslValue::String(x))) => Ok(Some(x.as_str())),
+            None => Ok(None),
+            _ => Err(FunctionErrorKind::WrongArgument(format!(
+                "Invalid value for {}",
+                name
+            ))),
+        };
+
+        // Login is optional. It must be later checked if the login was
         // already set by another option.
-        let login = match register.named("login") {
-            Some(ContextType::Value(NaslValue::String(x))) => Some(x.to_owned()),
-            None => None,
-            _ => {
-                return Err(FunctionErrorKind::WrongArgument(
-                    ("Invalid value for login").to_string(),
-                ))
-            }
-        };
-
-        let password = match register.named("password") {
-            Some(ContextType::Value(NaslValue::String(x))) => Some(x.as_str()),
-            None => None,
-            _ => {
-                return Err(FunctionErrorKind::WrongArgument(
-                    ("Invalid value for password").to_string(),
-                ))
-            }
-        };
-
-        let privatekey = match register.named("privatekey") {
-            Some(ContextType::Value(NaslValue::String(x))) => Some(x.as_str()),
-            None => None,
-            _ => {
-                return Err(FunctionErrorKind::WrongArgument(
-                    ("Invalid value for privatekey").to_string(),
-                ))
-            }
-        };
-        let passphrase = match register.named("passphrase") {
-            Some(ContextType::Value(NaslValue::String(x))) => Some(x.as_str()),
-            None => None,
-            _ => {
-                return Err(FunctionErrorKind::WrongArgument(
-                    ("Invalid invalid value for passphrase").to_string(),
-                ))
-            }
-        };
+        let login = get_named_val("login")?.map(str::to_string);
+        let password = get_named_val("password")?;
+        let privatekey = get_named_val("privatekey")?;
+        let passphrase = get_named_val("passphrase")?;
 
         if password.is_none() && privatekey.is_none() && passphrase.is_none() {
             //TODO: Get values from KB
@@ -910,20 +871,16 @@ impl Ssh {
                 if !session.user_set {
                     set_opt_user(session, login, session_id)?;
                 }
-                let verbose = session.verbose > 0;
 
                 // Get the authentication methods only once per session.
                 let methods: AuthMethods = {
                     if !session.authmethods_valid {
-                        get_authmethods(session, session_id, ctx.logger())?
+                        get_authmethods(session, session_id)?
                     } else {
                         session.authmethods
                     }
                 };
-                if verbose {
-                    ctx.logger()
-                        .info(&format!("Available methods:\n{:?}", methods));
-                }
+                debug!("Available methods:\n{:?}", methods);
 
                 if methods == AuthMethods::NONE {
                     return Ok(NaslValue::Number(0));
@@ -941,12 +898,10 @@ impl Ssh {
                             return Ok(NaslValue::Number(0));
                         }
                         Ok(_) => {
-                            if verbose {
-                                ctx.logger().info(&format!(
-                                    "SSH password authentication failed for session {}",
-                                    session_id
-                                ));
-                            }
+                            debug!(
+                                session_id = session_id,
+                                "SSH password authentication failed.",
+                            );
                         }
                         Err(_) => {
                             return Err(FunctionErrorKind::Dirty(format!(
@@ -973,13 +928,11 @@ impl Ssh {
                                         )));
                                         }
                                     };
-                                if verbose {
-                                    ctx.logger().info(&format!("SSH kbdint name={}", info.name));
-                                    ctx.logger().info(&format!(
-                                        "SSH kbdint instruction{}",
-                                        info.instruction
-                                    ));
-                                }
+                                debug!(
+                                    name = info.name,
+                                    instruction = info.instruction,
+                                    "SSH keyboard-interactive"
+                                );
 
                                 let mut answers: Vec<String> = Vec::new();
                                 for p in info.prompts.into_iter() {
@@ -1000,11 +953,10 @@ impl Ssh {
                                 }
                             }
                             Ok(_) => {
-                                if verbose {
-                                    ctx.logger().info(&format!(
-                                    "SSH keyboard-interactive authentication failed for session {}",
-                                    session_id));
-                                };
+                                debug!(
+                                    session_id = session_id,
+                                    "SSH keyboard-interactive authentication failed.",
+                                );
                                 continue;
                             }
                             Err(_) => {
@@ -1027,22 +979,16 @@ impl Ssh {
                                         return Ok(NaslValue::Number(0));
                                     }
                                     _ => {
-                                        if verbose {
-                                            ctx.logger().info(&format!("SSH authentication failed for session {}: No more authentication methods to try", session_id));
-                                        };
+                                        debug!(session_id=session_id, "SSH authentication failed. No more authentication methods to try");
                                     }
                                 }
                             }
                             _ => {
-                                if verbose {
-                                    ctx.logger().info(&format!("SSH public key authentication failed for session {}: Server does not want our key", session_id));
-                                };
+                                debug!(session_id=session_id, "SSH public key authentication failed.: Server does not want our key");
                             }
                         },
                         Err(_) => {
-                            if verbose {
-                                ctx.logger().info(&format!("SSH public key authentication failed for session {}: Error converting provided key", session.session_id));
-                            };
+                            debug!(session_id=session.session_id, "SSH public key authentication failed: Error converting provided key");
                         }
                     };
                 };
@@ -1098,10 +1044,10 @@ impl Ssh {
     ///    description.
     ///
     /// return A data block on success or NULL on error.
-    fn nasl_ssh_request_exec<K>(
+    fn nasl_ssh_request_exec(
         &self,
         register: &Register,
-        ctx: &Context<K>,
+        _: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1122,7 +1068,7 @@ impl Ssh {
 
         let cmd = match register.named("cmd") {
             Some(ContextType::Value(NaslValue::String(x))) => x,
-            _ => return Err(FunctionErrorKind::from("No command passed")),
+            _ => return Err(FunctionErrorKind::missing_argument("No command passed")),
         };
 
         let stdout = match register.named("stdout") {
@@ -1142,8 +1088,6 @@ impl Ssh {
             .find(|(_i, s)| s.session_id == session_id)
         {
             Some((_i, session)) => {
-                let verbose = session.verbose > 0;
-
                 if cmd.is_empty() {
                     return Ok(NaslValue::Null);
                 }
@@ -1165,15 +1109,8 @@ impl Ssh {
                     to_stderr = 0;
                 }
 
-                let (mut response, compat_buf) = exec_ssh_cmd(
-                    session,
-                    cmd,
-                    verbose,
-                    compat_mode,
-                    to_stdout,
-                    to_stderr,
-                    ctx.logger(),
-                )?;
+                let (mut response, compat_buf) =
+                    exec_ssh_cmd(session, cmd, compat_mode, to_stdout, to_stderr)?;
 
                 if compat_mode {
                     response.push_str(&compat_buf)
@@ -1198,10 +1135,10 @@ impl Ssh {
     /// - pty: To enable/disable the interactive shell. Default is 1 (interactive).
     ///
     /// @naslret An int on success or NULL on error.
-    fn nasl_ssh_shell_open<K>(
+    fn nasl_ssh_shell_open(
         &self,
         register: &Register,
-        _ctx: &Context<K>,
+        _ctx: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1271,13 +1208,13 @@ impl Ssh {
     ///
     /// nasl named params
     /// - timeout: Enable the blocking ssh read until it gives the timeout or there is no
-    /// bytes left to read.
+    ///   bytes left to read.
     ///
     /// return A string on success or NULL on error.
-    fn nasl_ssh_shell_read<K>(
+    fn nasl_ssh_shell_read(
         &self,
         register: &Register,
-        _ctx: &Context<K>,
+        _ctx: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1350,10 +1287,10 @@ impl Ssh {
     /// - cmd: A string to write to shell.
     ///
     /// return An integer: 0 on success, -1 on failure.
-    fn nasl_ssh_shell_write<K>(
+    fn nasl_ssh_shell_write(
         &self,
         register: &Register,
-        _ctx: &Context<K>,
+        _ctx: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1372,9 +1309,14 @@ impl Ssh {
             }
         };
 
+        let binding;
         let cmd = match register.named("cmd") {
             Some(ContextType::Value(NaslValue::String(x))) => x,
-            _ => return Err(FunctionErrorKind::from("cmd")),
+            Some(ContextType::Value(NaslValue::Data(x))) => {
+                binding = x.iter().map(|x| *x as char).collect::<String>();
+                &binding
+            }
+            _ => return Err(FunctionErrorKind::missing_argument("cmd")),
         };
 
         let mut sessions = lock_sessions(&self.sessions)?;
@@ -1398,7 +1340,7 @@ impl Ssh {
                     )));
                 }
 
-                match channel.stdin().write(cmd.as_bytes()) {
+                match channel.stdin().write_all(cmd.as_bytes()) {
                     Ok(_) => Ok(NaslValue::Number(0)),
                     Err(_) => Ok(NaslValue::Number(-1)),
                 }
@@ -1414,10 +1356,10 @@ impl Ssh {
     ///
     /// nasl params
     /// - An SSH session id.
-    fn nasl_ssh_shell_close<K>(
+    fn nasl_ssh_shell_close(
         &self,
         register: &Register,
-        _ctx: &Context<K>,
+        _ctx: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1475,10 +1417,10 @@ impl Ssh {
     /// - login: A string with the login name.
     ///  
     /// return A data block on success or NULL on error.
-    fn nasl_ssh_login_interactive<K>(
+    fn nasl_ssh_login_interactive(
         &self,
         register: &Register,
-        ctx: &Context<K>,
+        _: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1499,7 +1441,7 @@ impl Ssh {
 
         let login = match register.named("login") {
             Some(ContextType::Value(NaslValue::String(x))) => Some(x.to_owned()),
-            _ => return Err(FunctionErrorKind::from("login")),
+            _ => return Err(FunctionErrorKind::missing_argument("login")),
         };
 
         let mut sessions = lock_sessions(&self.sessions)?;
@@ -1512,20 +1454,17 @@ impl Ssh {
                 if !session.user_set {
                     set_opt_user(session, login, session_id)?;
                 }
-                let verbose = session.verbose > 0;
 
                 // Get the authentication methods only once per session.
                 let methods: AuthMethods = {
                     if !session.authmethods_valid {
-                        get_authmethods(session, session_id, ctx.logger())?
+                        get_authmethods(session, session_id)?
                     } else {
                         session.authmethods
                     }
                 };
-                if verbose {
-                    ctx.logger()
-                        .info(&format!("Available methods:\n{:?}", methods));
-                }
+                debug!("Available methods:\n{:?}", methods);
+
                 if methods.contains(AuthMethods::INTERACTIVE) {
                     let mut prompt = String::new();
                     loop {
@@ -1541,13 +1480,11 @@ impl Ssh {
                                         )));
                                         }
                                     };
-                                if verbose {
-                                    ctx.logger().info(&format!("SSH kbdint name={}", info.name));
-                                    ctx.logger().info(&format!(
-                                        "SSH kbdint instruction{}",
-                                        info.instruction
-                                    ));
-                                }
+                                debug!(
+                                    name = info.name,
+                                    instruction = info.instruction,
+                                    "SSH keyboard-interactive"
+                                );
 
                                 for p in info.prompts.into_iter() {
                                     if !p.echo {
@@ -1557,11 +1494,10 @@ impl Ssh {
                                 break;
                             }
                             Ok(_) => {
-                                if verbose {
-                                    ctx.logger().info(&format!(
+                                debug!(
                                     "SSH keyboard-interactive authentication failed for session {}",
-                                    session_id));
-                                };
+                                    session_id
+                                );
                                 continue;
                             }
                             Err(_) => {
@@ -1602,10 +1538,10 @@ impl Ssh {
     ///
     /// return An integer as status value; 0 indicates success.
     ///
-    fn nasl_ssh_login_interactive_pass<K>(
+    fn nasl_ssh_login_interactive_pass(
         &self,
         register: &Register,
-        ctx: &Context<K>,
+        _: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1626,7 +1562,7 @@ impl Ssh {
 
         let password = match register.named("pass") {
             Some(ContextType::Value(NaslValue::String(x))) => x,
-            _ => return Err(FunctionErrorKind::from("pass")),
+            _ => return Err(FunctionErrorKind::missing_argument("pass")),
         };
 
         let mut sessions = lock_sessions(&self.sessions)?;
@@ -1636,7 +1572,6 @@ impl Ssh {
             .find(|(_i, s)| s.session_id == session_id)
         {
             Some((_i, session)) => {
-                let verbose = session.verbose > 0;
                 let info = match session.session.userauth_keyboard_interactive_info() {
                     Ok(i) => i,
                     Err(_) => {
@@ -1650,11 +1585,11 @@ impl Ssh {
                     }
                 };
 
-                if verbose {
-                    ctx.logger().info(&format!("SSH kbdint name={}", info.name));
-                    ctx.logger()
-                        .info(&format!("SSH kbdint instruction{}", info.instruction));
-                }
+                debug!(
+                    name = info.name,
+                    instruction = info.instruction,
+                    "SSH keyboard-interactive"
+                );
 
                 let mut answers: Vec<String> = Vec::new();
                 for p in info.prompts.into_iter() {
@@ -1715,10 +1650,10 @@ impl Ssh {
     ///
     /// return A data block on success or NULL on error.
     ///
-    fn nasl_ssh_get_issue_banner<K>(
+    fn nasl_ssh_get_issue_banner(
         &self,
         register: &Register,
-        ctx: &Context<K>,
+        _: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1750,7 +1685,7 @@ impl Ssh {
                 }
 
                 if !session.authmethods_valid {
-                    get_authmethods(session, session_id, ctx.logger())?;
+                    get_authmethods(session, session_id)?;
                 }
 
                 match session.session.get_issue_banner() {
@@ -1775,10 +1710,10 @@ impl Ssh {
     /// - An SSH session id.
     ///
     /// return A data block on success or NULL on error.
-    fn nasl_ssh_get_server_banner<K>(
+    fn nasl_ssh_get_server_banner(
         &self,
         register: &Register,
-        _: &Context<K>,
+        _: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1827,10 +1762,10 @@ impl Ssh {
     /// - An SSH session id.
     ///
     /// return A string on success or NULL on error.
-    fn nasl_ssh_get_auth_methods<K>(
+    fn nasl_ssh_get_auth_methods(
         &self,
         register: &Register,
-        ctx: &Context<K>,
+        _: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1862,7 +1797,7 @@ impl Ssh {
                 }
 
                 if !session.authmethods_valid {
-                    get_authmethods(session, session_id, ctx.logger())?;
+                    get_authmethods(session, session_id)?;
                 };
 
                 let mut methods = vec![];
@@ -1903,10 +1838,10 @@ impl Ssh {
     /// - An SSH session id.
     ///
     /// @naslret A data block on success or NULL on error.
-    fn nasl_ssh_get_host_key<K>(
+    fn nasl_ssh_get_host_key(
         &self,
         register: &Register,
-        _: &Context<K>,
+        _: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1957,10 +1892,65 @@ impl Ssh {
     /// return An integer: 0 on success, -1 (SSH_ERROR) on Channel request
     /// subsystem failure. Greater than 0 means an error during SFTP init. NULL
     /// indicates a failure during session id verification.
-    fn nasl_sftp_enabled_check<K>(
+    fn nasl_sftp_enabled_check(
         &self,
         register: &Register,
-        ctx: &Context<K>,
+        _: &Context,
+    ) -> Result<NaslValue, FunctionErrorKind> {
+        let positional = register.positional();
+        if positional.is_empty() {
+            return Err(FunctionErrorKind::MissingPositionalArguments {
+                expected: 0,
+                got: 1,
+            });
+        }
+
+        let session_id = match &positional[0] {
+            NaslValue::Number(x) => *x as i32,
+            _ => {
+                return Err(FunctionErrorKind::WrongArgument(
+                    ("Invalid session ID").to_string(),
+                ))
+            }
+        };
+
+        let mut sessions = lock_sessions(&self.sessions)?;
+        match sessions
+            .iter_mut()
+            .enumerate()
+            .find(|(_i, s)| s.session_id == session_id)
+        {
+            Some((_i, session)) => match session.session.sftp() {
+                Ok(_) => Ok(NaslValue::Number(0)),
+                Err(e) => {
+                    debug!("SFTP enabled check error: {}", e);
+
+                    Ok(NaslValue::Number(1))
+                }
+            },
+            _ => Err(FunctionErrorKind::Dirty(format!(
+                "Session ID {} not found",
+                session_id
+            ))),
+        }
+    }
+
+    /// NASL NETCONF
+    ///
+
+    /// Execute the NETCONF subsystem on the the ssh channel
+    ///
+    /// nasluparam
+    /// - An SSH session id.
+    ///
+    /// naslret An int on success or NULL on error.
+    ///
+    /// param[in] lexic Lexical context of NASL interpreter.
+    /// return Session ID on success, NULL on failure.
+    fn nasl_ssh_execute_netconf_subsystem(
+        &self,
+        register: &Register,
+        _ctx: &Context,
     ) -> Result<NaslValue, FunctionErrorKind> {
         let positional = register.positional();
         if positional.is_empty() {
@@ -1986,17 +1976,39 @@ impl Ssh {
             .find(|(_i, s)| s.session_id == session_id)
         {
             Some((_i, session)) => {
-                let verbose = session.verbose > 0;
-                match session.session.sftp() {
-                    Ok(_) => Ok(NaslValue::Number(0)),
+                // new channel
+                let channel = match session.session.new_channel() {
+                    Ok(c) => c,
                     Err(e) => {
-                        if verbose {
-                            ctx.logger()
-                                .info(&format!("SFTP enabled check error: {}", e));
-                        }
-                        Ok(NaslValue::Number(1))
+                        return Err(FunctionErrorKind::Dirty(format!(
+                            "Failed to open a new channel for session ID {}: {}",
+                            session.session_id, e
+                        )));
                     }
-                }
+                };
+
+                match channel.open_session() {
+                    Ok(_) => (),
+                    Err(e) => {
+                        return Err(FunctionErrorKind::Dirty(format!(
+                            "Channel failed to open session for session ID {}: {}",
+                            session.session_id, e
+                        )));
+                    }
+                };
+
+                match channel.request_subsystem("netconf") {
+                    Ok(_) => (),
+                    Err(e) => {
+                        return Err(FunctionErrorKind::Dirty(format!(
+                            "Channel failed to execyte NETCONF subsystem for session ID {}: {}",
+                            session.session_id, e
+                        )));
+                    }
+                };
+
+                session.channel = Some(channel);
+                Ok(NaslValue::Number(session_id as i64))
             }
             _ => Err(FunctionErrorKind::Dirty(format!(
                 "Session ID {} not found",
@@ -2005,7 +2017,7 @@ impl Ssh {
         }
     }
 
-    fn lookup<K>(key: &str) -> Option<NaslSSHFunction<K>> {
+    fn lookup(key: &str) -> Option<NaslSSHFunction> {
         match key {
             "ssh_connect" => Some(Ssh::nasl_ssh_connect),
             "ssh_disconnect" => Some(Ssh::nasl_ssh_disconnect),
@@ -2025,12 +2037,13 @@ impl Ssh {
             "ssh_get_auth_methods" => Some(Ssh::nasl_ssh_get_auth_methods),
             "ssh_get_host_key" => Some(Ssh::nasl_ssh_get_host_key),
             "sftp_enabled_check" => Some(Ssh::nasl_sftp_enabled_check),
+            "ssh_execute_netconf_subsystem" => Some(Ssh::nasl_ssh_execute_netconf_subsystem),
             _ => None,
         }
     }
 }
 
-impl<K: AsRef<str>> nasl_builtin_utils::NaslFunctionExecuter<K> for Ssh {
+impl nasl_builtin_utils::NaslFunctionExecuter for Ssh {
     fn nasl_fn_cache_clear(&self) -> Option<usize> {
         let mut data = Arc::as_ref(&self.sessions).lock().unwrap();
         if data.is_empty() {
@@ -2046,12 +2059,12 @@ impl<K: AsRef<str>> nasl_builtin_utils::NaslFunctionExecuter<K> for Ssh {
         &self,
         name: &str,
         register: &Register,
-        context: &Context<K>,
+        context: &Context,
     ) -> Option<nasl_builtin_utils::NaslResult> {
         Ssh::lookup(name).map(|f| f(self, register, context))
     }
 
     fn nasl_fn_defined(&self, name: &str) -> bool {
-        Ssh::lookup::<K>(name).is_some()
+        Ssh::lookup(name).is_some()
     }
 }
