@@ -1,10 +1,14 @@
+// SPDX-FileCopyrightText: 2024 Greenbone AG
+//
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
+
 #![doc = include_str!("../README.md")]
 #![warn(missing_docs)]
 
 use nasl_builtin_utils::{Context, NaslFunctionRegister, NaslVarRegister, Register};
-use nasl_syntax::{logger::NaslLogger, Loader};
-use storage::{DefaultDispatcher, Storage};
+use storage::{ContextKey, DefaultDispatcher};
 mod array;
+mod report_functions;
 
 /// The description builtin function
 ///
@@ -15,18 +19,18 @@ mod array;
 /// script. This is handled within the `nasl_interpreter::Interpreter`.
 pub struct Std;
 
-impl<K: AsRef<str>> nasl_builtin_utils::NaslFunctionExecuter<K> for Std {
+impl nasl_builtin_utils::NaslFunctionExecuter for Std {
     fn nasl_fn_execute(
         &self,
         name: &str,
         register: &Register,
-        context: &Context<K>,
+        context: &Context,
     ) -> Option<nasl_builtin_utils::NaslResult> {
         array::lookup(name).map(|x| x(register, context))
     }
 
     fn nasl_fn_defined(&self, name: &str) -> bool {
-        array::lookup::<K>(name).is_some()
+        array::lookup(name).is_some()
     }
 }
 
@@ -46,16 +50,16 @@ impl<K: AsRef<str>> nasl_builtin_utils::NaslFunctionExecuter<K> for Std {
 ///
 /// ```
 /// #[cfg(not(feature = "nasl-builtin-ssh"))]
-/// fn add_ssh<K: AsRef<str>>(
-///     builder: nasl_builtin_utils::NaslfunctionRegisterBuilder<K>,
-/// ) -> nasl_builtin_utils::NaslfunctionRegisterBuilder<K> {
+/// fn add_ssh(
+///     builder: nasl_builtin_utils::NaslfunctionRegisterBuilder,
+/// ) -> nasl_builtin_utils::NaslfunctionRegisterBuilder {
 ///     builder
 /// }
 ///
 /// #[cfg(feature = "nasl-builtin-ssh")]
-/// fn add_ssh<K: AsRef<str>>(
-///     builder: nasl_builtin_utils::NaslfunctionRegisterBuilder<K>,
-/// ) -> nasl_builtin_utils::NaslfunctionRegisterBuilder<K> {
+/// fn add_ssh(
+///     builder: nasl_builtin_utils::NaslfunctionRegisterBuilder,
+/// ) -> nasl_builtin_utils::NaslfunctionRegisterBuilder {
 ///     builder.push_register(nasl_builtin_ssh::Ssh::default())
 /// }
 ///
@@ -64,15 +68,21 @@ impl<K: AsRef<str>> nasl_builtin_utils::NaslFunctionExecuter<K> for Std {
 /// ```text
 /// builder = add_ssh(builder);
 /// ```
-pub fn nasl_std_functions<K: AsRef<str>>() -> nasl_builtin_utils::NaslFunctionRegister<K> {
+pub fn nasl_std_functions() -> nasl_builtin_utils::NaslFunctionRegister {
     let mut builder = nasl_builtin_utils::NaslfunctionRegisterBuilder::new()
         .push_register(Std)
+        .push_register(report_functions::Reporting::default())
         .push_register(nasl_builtin_knowledge_base::KnowledgeBase)
         .push_register(nasl_builtin_misc::Misc)
         .push_register(nasl_builtin_string::NaslString)
         .push_register(nasl_builtin_host::Host)
+        .push_register(nasl_builtin_http::NaslHttp::default())
+        .push_register(nasl_builtin_network::socket::NaslSockets::default())
+        .push_register(nasl_builtin_network::network::Network)
+        .push_register(nasl_builtin_regex::RegularExpressions)
         .push_register(nasl_builtin_cryptographic::Cryptographic)
         .push_register(nasl_builtin_description::Description);
+
     builder = add_ssh(builder);
     builder = add_raw_ip(builder);
     builder.build()
@@ -118,16 +128,16 @@ pub fn nasl_std_variables() -> NaslVarRegister {
 }
 
 #[cfg(not(feature = "nasl-builtin-ssh"))]
-fn add_ssh<K: AsRef<str>>(
-    builder: nasl_builtin_utils::NaslfunctionRegisterBuilder<K>,
-) -> nasl_builtin_utils::NaslfunctionRegisterBuilder<K> {
+fn add_ssh(
+    builder: nasl_builtin_utils::NaslfunctionRegisterBuilder,
+) -> nasl_builtin_utils::NaslfunctionRegisterBuilder {
     builder
 }
 
 #[cfg(feature = "nasl-builtin-raw-ip")]
-fn add_raw_ip<K: AsRef<str>>(
-    builder: nasl_builtin_utils::NaslfunctionRegisterBuilder<K>,
-) -> nasl_builtin_utils::NaslfunctionRegisterBuilder<K> {
+fn add_raw_ip(
+    builder: nasl_builtin_utils::NaslfunctionRegisterBuilder,
+) -> nasl_builtin_utils::NaslfunctionRegisterBuilder {
     builder.push_register(nasl_builtin_raw_ip::RawIp)
 }
 
@@ -139,16 +149,16 @@ fn add_raw_ip_vars(
 }
 
 #[cfg(feature = "nasl-builtin-ssh")]
-fn add_ssh<K: AsRef<str>>(
-    builder: nasl_builtin_utils::NaslfunctionRegisterBuilder<K>,
-) -> nasl_builtin_utils::NaslfunctionRegisterBuilder<K> {
+fn add_ssh(
+    builder: nasl_builtin_utils::NaslfunctionRegisterBuilder,
+) -> nasl_builtin_utils::NaslfunctionRegisterBuilder {
     builder.push_register(nasl_builtin_ssh::Ssh::default())
 }
 
 #[cfg(not(feature = "nasl-builtin-raw-ip"))]
-fn add_raw_ip<K: AsRef<str>>(
-    builder: nasl_builtin_utils::NaslfunctionRegisterBuilder<K>,
-) -> nasl_builtin_utils::NaslfunctionRegisterBuilder<K> {
+fn add_raw_ip(
+    builder: nasl_builtin_utils::NaslfunctionRegisterBuilder,
+) -> nasl_builtin_utils::NaslfunctionRegisterBuilder {
     builder
 }
 
@@ -159,97 +169,60 @@ fn add_raw_ip_vars(
     builder
 }
 
-/// Contains the key as well as the dispatcher.
-///
-/// This is to ensure that the key and the dispatcher do have the same type.
-/// It makes it also a bit easier to use as the key as well as the dispatcher are usually created
-/// together.
-pub struct KeyDispatcherSet<K: AsRef<str>> {
-    key: K,
-    storage: Box<dyn Storage<K>>,
-}
-
 /// The context builder.
 ///
 /// This is the main entry point for the nasl interpreter and adds all the functions defined in
 /// [nasl_std_functions] to functions register.
-pub struct ContextBuilder<K: AsRef<str>, S> {
-    /// The key and dispatcher set.
-    pub key: S,
-    /// The target to test.
-    pub target: String,
+// TODO: remove key and target and box dyn
+pub struct ContextFactory<Loader, Storage> {
+    /// The shared storage
+    pub storage: Storage,
     /// The loader to load the nasl files.
-    pub loader: Box<dyn Loader>,
-    /// The logger to log.
-    pub logger: Box<dyn NaslLogger>,
+    pub loader: Loader,
     /// The functions available to the nasl script.
-    pub functions: NaslFunctionRegister<K>,
+    pub functions: NaslFunctionRegister,
 }
 
-impl Default for ContextBuilder<String, KeyDispatcherSet<String>> {
+impl Default for ContextFactory<nasl_syntax::NoOpLoader, storage::DefaultDispatcher> {
     fn default() -> Self {
         Self {
-            key: KeyDispatcherSet {
-                key: Default::default(),
-                storage: Box::<DefaultDispatcher<String>>::default(),
-            },
-            target: Default::default(),
-            loader: Default::default(),
-            logger: Default::default(),
+            loader: nasl_syntax::NoOpLoader::default(),
             functions: nasl_std_functions(),
+            storage: DefaultDispatcher::default(),
         }
     }
 }
 
-impl<K: AsRef<str>, S> ContextBuilder<K, S> {
-    /// Sets the target to test.
-    pub fn target(mut self, target: String) -> Self {
-        self.target = target;
-        self
-    }
-
-    /// Sets the loader to load the nasl files.
-    pub fn loader<L: Loader + 'static>(mut self, loader: L) -> Self {
-        self.loader = Box::new(loader);
-        self
-    }
-
-    /// Sets the logger to log.
-    pub fn logger<L: NaslLogger + 'static>(mut self, logger: L) -> Self {
-        self.logger = Box::new(logger);
-        self
+impl<L, S> ContextFactory<L, S>
+where
+    L: nasl_syntax::Loader,
+    S: storage::Storage,
+{
+    /// Creates a new ContextFactory with nasl_std_functions
+    ///
+    /// If you want to override the functions register please use functions method.
+    pub fn new(loader: L, storage: S) -> ContextFactory<L, S> {
+        ContextFactory {
+            storage,
+            loader,
+            functions: nasl_std_functions(),
+        }
     }
 
     /// Sets the functions available to the nasl script.
-    pub fn functions(mut self, functions: NaslFunctionRegister<K>) -> Self {
+    pub fn functions(mut self, functions: NaslFunctionRegister) -> Self {
         self.functions = functions;
         self
     }
-}
 
-impl<K: AsRef<str>> ContextBuilder<K, KeyDispatcherSet<K>> {
-    /// Creates a new context builder with the given key and storage.
-    pub fn new(key: K, storage: Box<dyn Storage<K>>) -> Self {
-        Self {
-            key: KeyDispatcherSet { key, storage },
-            target: Default::default(),
-            loader: Default::default(),
-            logger: Default::default(),
-            functions: nasl_std_functions(),
-        }
-    }
-
-    /// Createz the context.
-    ///
-    /// Be aware that unlike normal builder, because of the lifetime of the dispatcher, the ContextBuilder must exist as long as the context and cannot be dropped immediately.
-    pub fn build(&self) -> Context<K> {
+    /// Creates a new Context with the shared loader, logger and function register
+    pub fn build(&self, key: ContextKey, target: String) -> Context {
         Context::new(
-            &self.key.key,
-            &self.target,
-            self.key.storage.as_dispatcher(),
-            self.key.storage.as_retriever(),
-            &*self.loader,
-            self.logger.as_ref(),
+            key,
+            target,
+            self.storage.as_dispatcher(),
+            self.storage.as_retriever(),
+            &self.loader,
             &self.functions,
         )
     }
