@@ -1,6 +1,6 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 use std::collections::HashMap;
 
@@ -9,7 +9,7 @@ use nasl_syntax::{AssignOrder, Statement, TokenCategory};
 use crate::{error::InterpretError, interpreter::InterpretResult, Interpreter};
 use nasl_builtin_utils::ContextType;
 use nasl_syntax::NaslValue;
-use Statement::*;
+use nasl_syntax::StatementKind::*;
 
 /// Is a trait to handle function assignments within nasl.
 pub(crate) trait AssignExtension {
@@ -51,18 +51,15 @@ fn prepare_dict(left: NaslValue) -> HashMap<String, NaslValue> {
     }
 }
 
-impl<'a, K> Interpreter<'a, K>
-where
-    K: AsRef<str>,
-{
+impl<'a> Interpreter<'a> {
     fn save(&mut self, idx: usize, key: &str, value: NaslValue) {
-        self.registrat
+        self.register_mut()
             .add_to_index(idx, key, ContextType::Value(value));
     }
 
     fn named_value(&self, key: &str) -> Result<(usize, NaslValue), InterpretError> {
         match self
-            .registrat()
+            .register()
             .index_named(key)
             .unwrap_or((0, &ContextType::Value(NaslValue::Null)))
         {
@@ -186,10 +183,7 @@ where
     }
 }
 
-impl<'a, K> AssignExtension for Interpreter<'a, K>
-where
-    K: AsRef<str>,
-{
+impl<'a> AssignExtension for Interpreter<'a> {
     fn assign(
         &mut self,
         category: &TokenCategory,
@@ -198,12 +192,13 @@ where
         right: &Statement,
     ) -> InterpretResult {
         let (key, lookup) = {
-            match left {
-                Variable(ref token) => (Self::identifier(token)?, None),
-                Array(ref token, Some(stmt), _) => {
-                    (Self::identifier(token)?, Some(self.resolve(stmt)?))
-                }
-                Array(ref token, None, _) => (Self::identifier(token)?, None),
+            match left.kind() {
+                Variable => (Self::identifier(left.as_token())?, None),
+                Array(Some(stmt)) => (
+                    Self::identifier(left.as_token())?,
+                    Some(self.resolve(stmt)?),
+                ),
+                Array(None) => (Self::identifier(left.as_token())?, None),
                 _ => return Err(InterpretError::unsupported(left, "Array or Variable")),
             }
         };
@@ -275,12 +270,10 @@ mod tests {
         a--;
         --a;
         "###;
-        let mut register = Register::default();
-        let binding = ContextBuilder::default();
-        let context = binding.build();
-        let mut interpreter = Interpreter::new(&mut register, &context);
-        let mut parser =
-            parse(code).map(|x| interpreter.resolve(&x.expect("no parse error expected")));
+        let register = Register::default();
+        let binding = ContextFactory::default();
+        let context = binding.build(Default::default(), Default::default());
+        let mut parser = CodeInterpreter::new(code, register, &context);
         assert_eq!(parser.next(), Some(Ok(12.into())));
         assert_eq!(parser.next(), Some(Ok(25.into())));
         assert_eq!(parser.next(), Some(Ok(23.into())));
@@ -310,12 +303,10 @@ mod tests {
         a[0]++;
         ++a[0];
         "###;
-        let mut register = Register::default();
-        let binding = ContextBuilder::default();
-        let context = binding.build();
-        let mut interpreter = Interpreter::new(&mut register, &context);
-        let mut parser =
-            parse(code).map(|x| interpreter.resolve(&x.expect("no parse error expected")));
+        let register = Register::default();
+        let binding = ContextFactory::default();
+        let context = binding.build(Default::default(), Default::default());
+        let mut parser = CodeInterpreter::new(code, register, &context);
         assert_eq!(parser.next(), Some(Ok(12.into())));
         assert_eq!(parser.next(), Some(Ok(25.into())));
         assert_eq!(parser.next(), Some(Ok(23.into())));
@@ -334,12 +325,10 @@ mod tests {
         a[2] = 12;
         a;
         "###;
-        let mut register = Register::default();
-        let binding = ContextBuilder::default();
-        let context = binding.build();
-        let mut interpreter = Interpreter::new(&mut register, &context);
-        let mut parser =
-            parse(code).map(|x| interpreter.resolve(&x.expect("no parse error expected")));
+        let register = Register::default();
+        let binding = ContextFactory::default();
+        let context = binding.build(Default::default(), Default::default());
+        let mut parser = CodeInterpreter::new(code, register, &context);
         assert_eq!(parser.next(), Some(Ok(12.into())));
         assert_eq!(
             parser.next(),
@@ -359,12 +348,10 @@ mod tests {
         a[2] = 12;
         a;
         "###;
-        let mut register = Register::default();
-        let binding = ContextBuilder::default();
-        let context = binding.build();
-        let mut interpreter = Interpreter::new(&mut register, &context);
-        let mut parser =
-            parse(code).map(|x| interpreter.resolve(&x.expect("no parse error expected")));
+        let register = Register::default();
+        let binding = ContextFactory::default();
+        let context = binding.build(Default::default(), Default::default());
+        let mut parser = CodeInterpreter::new(code, register, &context);
         assert_eq!(parser.next(), Some(Ok(12.into())));
         assert_eq!(parser.next(), Some(Ok(12.into())));
         assert_eq!(parser.next(), Some(Ok(12.into())));
@@ -385,12 +372,10 @@ mod tests {
         a;
         a['hi'];
         "###;
-        let mut register = Register::default();
-        let binding = ContextBuilder::default();
-        let context = binding.build();
-        let mut interpreter = Interpreter::new(&mut register, &context);
-        let mut parser =
-            parse(code).map(|x| interpreter.resolve(&x.expect("no parse error expected")));
+        let register = Register::default();
+        let binding = ContextFactory::default();
+        let context = binding.build(Default::default(), Default::default());
+        let mut parser = CodeInterpreter::new(code, register, &context);
         assert_eq!(parser.next(), Some(Ok(12.into())));
         assert_eq!(
             parser.next(),
@@ -407,12 +392,10 @@ mod tests {
         let code = r###"
         a = [1, 2, 3];
         "###;
-        let mut register = Register::default();
-        let binding = ContextBuilder::default();
-        let context = binding.build();
-        let mut interpreter = Interpreter::new(&mut register, &context);
-        let mut parser =
-            parse(code).map(|x| interpreter.resolve(&x.expect("no parse error expected")));
+        let register = Register::default();
+        let binding = ContextFactory::default();
+        let context = binding.build(Default::default(), Default::default());
+        let mut parser = CodeInterpreter::new(code, register, &context);
         assert_eq!(
             parser.next(),
             Some(Ok(NaslValue::Array(vec![1.into(), 2.into(), 3.into()])))
