@@ -1,8 +1,8 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
-use nasl_syntax::{Statement, Token, TokenCategory};
+use nasl_syntax::{Statement, StatementKind, Token, TokenCategory};
 
 use crate::{error::InterpretError, interpreter::InterpretResult, Interpreter};
 use nasl_builtin_utils::ContextType;
@@ -18,10 +18,7 @@ pub(crate) trait DeclareFunctionExtension {
     ) -> InterpretResult;
 }
 
-impl<'a, K> DeclareFunctionExtension for Interpreter<'a, K>
-where
-    K: AsRef<str>,
-{
+impl<'a> DeclareFunctionExtension for Interpreter<'a> {
     fn declare_function(
         &mut self,
         name: &Token,
@@ -31,15 +28,15 @@ where
         let name = &Self::identifier(name)?;
         let mut names = vec![];
         for a in arguments {
-            match a {
-                Statement::Variable(token) => {
-                    let param_name = &Self::identifier(token)?;
+            match a.kind() {
+                StatementKind::Variable => {
+                    let param_name = &Self::identifier(a.as_token())?;
                     names.push(param_name.to_owned());
                 }
                 _ => return Err(InterpretError::unsupported(a, "variable")),
             }
         }
-        self.registrat
+        self.register_mut()
             .add_global(name, ContextType::Function(names, execution.clone()));
         Ok(NaslValue::Null)
     }
@@ -49,16 +46,16 @@ pub(crate) trait DeclareVariableExtension {
     fn declare_variable(&mut self, scope: &Token, stmts: &[Statement]) -> InterpretResult;
 }
 
-impl<'a, K> DeclareVariableExtension for Interpreter<'a, K> {
+impl<'a> DeclareVariableExtension for Interpreter<'a> {
     fn declare_variable(&mut self, scope: &Token, stmts: &[Statement]) -> InterpretResult {
         let mut add = |key: &str| {
             let value = ContextType::Value(NaslValue::Null);
             match scope.category() {
                 TokenCategory::Identifier(nasl_syntax::IdentifierType::GlobalVar) => {
-                    self.registrat.add_global(key, value)
+                    self.register_mut().add_global(key, value)
                 }
                 TokenCategory::Identifier(nasl_syntax::IdentifierType::LocalVar) => {
-                    self.registrat.add_local(key, value)
+                    self.register_mut().add_local(key, value)
                 }
                 _ => unreachable!(
                     "{} should not be identified as an declare statement",
@@ -68,8 +65,8 @@ impl<'a, K> DeclareVariableExtension for Interpreter<'a, K> {
         };
 
         for stmt in stmts {
-            if let Statement::Variable(ref token) = stmt {
-                if let TokenCategory::Identifier(name) = token.category() {
+            if let StatementKind::Variable = stmt.kind() {
+                if let TokenCategory::Identifier(name) = stmt.as_token().category() {
                     add(&name.to_string());
                 }
             };
@@ -93,12 +90,10 @@ mod tests {
         test(a: 1, b: 2);
         c;
         "###;
-        let mut register = Register::default();
-        let binding = ContextBuilder::default();
-        let context = binding.build();
-        let mut interpreter = Interpreter::new(&mut register, &context);
-        let mut parser =
-            parse(code).map(|x| interpreter.resolve(&x.expect("unexpected parse error")));
+        let register = Register::default();
+        let binding = ContextFactory::default();
+        let context = binding.build(Default::default(), Default::default());
+        let mut parser = CodeInterpreter::new(code, register, &context);
         assert_eq!(parser.next(), Some(Ok(NaslValue::Null)));
         assert_eq!(parser.next(), Some(Ok(3.into())));
         assert!(matches!(parser.next(), Some(Ok(NaslValue::Null)))); // not found
@@ -112,12 +107,10 @@ mod tests {
         }
         test(a: 1, b: 2);
         "###;
-        let mut register = Register::default();
-        let binding = ContextBuilder::default();
-        let context = binding.build();
-        let mut interpreter = Interpreter::new(&mut register, &context);
-        let mut parser =
-            parse(code).map(|x| interpreter.resolve(&x.expect("unexpected parse error")));
+        let register = Register::default();
+        let binding = ContextFactory::default();
+        let context = binding.build(Default::default(), Default::default());
+        let mut parser = CodeInterpreter::new(code, register, &context);
         assert_eq!(parser.next(), Some(Ok(NaslValue::Null)));
         assert_eq!(parser.next(), Some(Ok(3.into())));
     }
