@@ -1,21 +1,24 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
-use std::{fmt::Display, io};
+use std::io;
 
 use nasl_builtin_utils::error::FunctionErrorKind;
 use nasl_syntax::{Statement, SyntaxError, TokenCategory};
 use storage::StorageError;
 
 use nasl_syntax::LoadError;
+use thiserror::Error;
 
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Error)]
 /// An error that occurred while calling a function
+#[error("Error while calling function '{function}': {kind}")]
 pub struct FunctionError {
     /// Name of the function
     pub function: String,
     /// Kind of error
+    #[source]
     pub kind: FunctionErrorKind,
 }
 
@@ -29,35 +32,37 @@ impl FunctionError {
     }
 }
 
-impl Display for FunctionError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}: {}", self.function, self.kind)
-    }
-}
-
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Error)]
 /// Is used to represent an error while interpreting
+#[error("{}{kind}", self.origin.clone().map(|e| format!("{e}: ")).unwrap_or_default())]
 pub struct InterpretError {
     /// Defined the type of error that occurred.
+    #[source]
     pub kind: InterpretErrorKind,
     /// The statement on which this error occurred.
     pub origin: Option<Statement>,
 }
 
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Error)]
 /// Is used to give hints to the user how to react on an error while interpreting
 pub enum InterpretErrorKind {
     /// When returned context is a function when a value is required.
+    #[error("Expected a value but got a function.")]
     FunctionExpectedValue,
     /// When returned context is a value when a function is required.
+    #[error("Expected a function but got a value.")]
     ValueExpectedFunction,
     /// When a specific type is expected
+    #[error("Expected the type {0}")]
     WrongType(String),
     /// When a specific token category is required but not given.
+    #[error("Expected the category {0}")]
     WrongCategory(TokenCategory),
     /// Regex parsing went wrong.
+    #[error("Invalid regular expression: {0}")]
     InvalidRegex(String),
-    /// An SyntaxError while including another script
+    /// A SyntaxError while including another script
+    #[error("Error while including file {filename}{}: {err}", {err}.as_token() .map(|t| format!(", line: {}, col: {}", t.position.0, t.position.1)) .unwrap_or_default())]
     IncludeSyntaxError {
         /// The name of the file trying to include
         filename: String,
@@ -65,68 +70,29 @@ pub enum InterpretErrorKind {
         err: SyntaxError,
     },
     /// SyntaxError
+    #[error("{0}")]
     SyntaxError(SyntaxError),
     /// When the given key was not found in the context
+    #[error("Key not found: {0}")]
     NotFound(String),
     /// A StorageError occurred
     // FIXME rename to general error
+    #[error("{0}")]
     StorageError(StorageError),
     /// A LoadError occurred
+    #[error("{0}")]
     LoadError(LoadError),
     /// A Formatting error occurred
+    #[error("{0}")]
     FMTError(std::fmt::Error),
     /// An IOError occurred
+    #[error("{0}")]
     IOError(io::ErrorKind),
     /// An error occurred while calling a built-in function.
+    #[error("{0}")]
     FunctionCallError(FunctionError),
 }
 
-impl Display for InterpretErrorKind {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        match self {
-            InterpretErrorKind::FunctionExpectedValue => {
-                write!(f, "expected a value but got a function")
-            }
-            InterpretErrorKind::ValueExpectedFunction => {
-                write!(f, "expected a function but got a value")
-            }
-            InterpretErrorKind::WrongType(e) => write!(f, "expected the type {e}"),
-            InterpretErrorKind::WrongCategory(e) => write!(f, "expecteced category {e}"),
-            InterpretErrorKind::InvalidRegex(e) => write!(f, "regular expression: {e} is invalid"),
-            InterpretErrorKind::IncludeSyntaxError { filename, err } => {
-                write!(
-                    f,
-                    "on include {filename}{}: {err}",
-                    err.as_token()
-                        .map(|t| format!(", line: {}, col: {}", t.position.0, t.position.1))
-                        .unwrap_or_default()
-                )
-            }
-            InterpretErrorKind::SyntaxError(e) => write!(f, "{e}"),
-            InterpretErrorKind::NotFound(e) => write!(f, "{e} not found"),
-            InterpretErrorKind::StorageError(e) => write!(f, "{e}"),
-            InterpretErrorKind::LoadError(e) => write!(f, "{e}"),
-            InterpretErrorKind::FMTError(e) => write!(f, "{e}"),
-            InterpretErrorKind::IOError(e) => write!(f, "{e}"),
-            InterpretErrorKind::FunctionCallError(e) => write!(f, "{e}"),
-        }
-    }
-}
-
-impl Display for InterpretError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(
-            f,
-            "{}{}",
-            self.origin
-                .clone()
-                .map(|e| format!("{e}: "))
-                .unwrap_or_default(),
-            self.kind
-        )
-    }
-}
-
 impl InterpretError {
     /// Creates a new Error with line and col set to 0
     ///
@@ -172,7 +138,7 @@ impl InterpretError {
     pub fn line_column(&self) -> (usize, usize) {
         self.origin
             .as_ref()
-            .and_then(|stmt| stmt.as_token())
+            .map(|stmt| stmt.as_token())
             .map(|x| x.line_column)
             .unwrap_or_default()
     }
@@ -270,17 +236,7 @@ impl From<FunctionError> for InterpretError {
             FunctionErrorKind::GeneralError(e) => {
                 Self::new(InterpretErrorKind::StorageError(e), None)
             }
-            FunctionErrorKind::MissingPositionalArguments {
-                expected: _,
-                got: _,
-            }
-            | FunctionErrorKind::MissingArguments(_)
-            | FunctionErrorKind::Infallible(_)
-            | FunctionErrorKind::WrongArgument(_)
-            | FunctionErrorKind::Dirty(_)
-            | FunctionErrorKind::Diagnostic(_, _) => {
-                Self::new(InterpretErrorKind::FunctionCallError(fe), None)
-            }
+            _ => Self::new(InterpretErrorKind::FunctionCallError(fe), None),
         }
     }
 }
