@@ -1,6 +1,6 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 //! Defines Operations used in Lexer to be transformed to Statements.
 use crate::token::{Category, IdentifierType, Token};
@@ -30,7 +30,7 @@ pub(crate) enum Operation {
 
 impl Operation {
     /// May create a new Operation based on given token. It returns None when the token.category is unknown.
-    pub(crate) fn new(token: Token) -> Option<Operation> {
+    pub(crate) fn new(token: &Token) -> Option<Operation> {
         match token.category() {
             Category::Plus
             | Category::Star
