@@ -1,6 +1,6 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 //! This module defines the TokenTypes as well as Token and extends Cursor with advance_token
 use std::fmt::Display;
@@ -406,6 +406,16 @@ pub struct Token {
     pub position: (usize, usize),
 }
 
+impl Default for Token {
+    fn default() -> Self {
+        Token {
+            category: Category::UnknownSymbol,
+            line_column: (0, 0),
+            position: (0, 0),
+        }
+    }
+}
+
 impl Token {
     /// Returns UnknownSymbol without line column or position
     pub fn unexpected_none() -> Self {
@@ -556,18 +566,11 @@ impl<'a> Tokenizer<'a> {
         if self.cursor.is_eof() {
             Category::Unclosed(UnclosedCategory::String)
         } else {
-            let mut result = self.code[Range {
+            let result = self.code[Range {
                 start,
                 end: self.cursor.len_consumed(),
             }]
             .to_owned();
-            result = result.replace(r"\n", "\n");
-            result = result.replace(r"\\", "\\");
-            result = result.replace(r#"\""#, "\"");
-            result = result.replace(r"\'", "'");
-            result = result.replace(r"\r", "\r");
-            result = result.replace(r"\t", "\t");
-            // skip ""
             self.cursor.advance();
             Category::String(result)
         }
@@ -589,15 +592,19 @@ impl<'a> Tokenizer<'a> {
         if self.cursor.is_eof() {
             Category::Unclosed(UnclosedCategory::Data)
         } else {
-            let raw = self.code[Range {
+            let mut raw_str = self.code[Range {
                 start,
                 end: self.cursor.len_consumed(),
             }]
-            .as_bytes()
-            .to_vec();
-
+            .to_owned();
+            raw_str = raw_str.replace(r#"\""#, "\"");
+            raw_str = raw_str.replace(r#"\n"#, "\n");
+            raw_str = raw_str.replace(r"\\", "\\");
+            raw_str = raw_str.replace(r"\'", "'");
+            raw_str = raw_str.replace(r"\r", "\r");
+            raw_str = raw_str.replace(r"\t", "\t");
             self.cursor.advance();
-            Category::Data(raw)
+            Category::Data(raw_str.as_bytes().to_vec())
         }
     }
     fn may_parse_ipv4(&mut self, base: Base, start: usize) -> Option<Category> {
@@ -888,7 +895,7 @@ mod tests {
     #[test]
     fn quotable_string() {
         verify_tokens!(
-            "'Hello \\'you\\'!'",
+            r#"'Hello \\\'you\\\'!'"#,
             ["[72, 101, 108, 108, 111, 32, 92, 39, 121, 111, 117, 92, 39, 33]"]
         );
         verify_tokens!("'Hello \\'you\\'!\\'", ["UnclosedData"]);
@@ -942,13 +949,21 @@ mod tests {
     #[test]
     fn string_quoting() {
         verify_tokens!(
-            r"'webapps\\appliance\\'",
+            r"'webapps\\\\appliance\\\\'",
             [
                 r"[119, 101, 98, 97, 112, 112, 115, 92, 92, 97, 112, 112, 108, 105, 97, 110, 99, 101, 92, 92]",
             ]
         );
     }
 
+    #[test]
+    fn data_escape_quoting() {
+        verify_tokens!(
+            r#"'version=\"1.0\"'"#,
+            [r"[118, 101, 114, 115, 105, 111, 110, 61, 34, 49, 46, 48, 34]",]
+        );
+    }
+
     #[test]
     fn simplified_ipv4_address() {
         verify_tokens!("10.187.76.12", ["10.187.76.12",]);
