@@ -1,6 +1,6 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 use std::{
     fmt::{self, Display, Formatter},
@@ -19,9 +19,87 @@ pub struct Feed {
     pub signature_check: bool,
 }
 
+#[derive(Deserialize, Serialize, Debug, Clone)]
+pub struct Notus {
+    pub products_path: PathBuf,
+    pub advisories_path: PathBuf,
+}
+
+#[derive(Deserialize, Serialize, Debug, Clone)]
+pub struct Redis {
+    pub url: String,
+}
+
+#[derive(Debug, Clone, serde::Serialize, serde::Deserialize)]
+pub struct Scheduler {
+    #[serde(default)]
+    pub max_queued_scans: Option<usize>,
+    #[serde(default)]
+    pub max_running_scans: Option<usize>,
+    #[serde(default)]
+    pub min_free_mem: Option<u64>,
+    pub check_interval: Duration,
+}
+
+impl Default for Scheduler {
+    fn default() -> Self {
+        Self {
+            check_interval: Duration::from_millis(500),
+            max_queued_scans: None,
+            max_running_scans: None,
+            min_free_mem: None,
+        }
+    }
+}
+
+#[derive(Deserialize, Serialize, Debug, Clone, Default)]
+pub struct Scanner {
+    #[serde(default, rename = "type")]
+    pub scanner_type: ScannerType,
+    #[serde(default)]
+    pub ospd: OspdWrapper,
+}
+
+#[derive(Deserialize, Serialize, Debug, Clone)]
+pub enum ScannerType {
+    #[serde(rename = "ospd")]
+    OSPD,
+    #[serde(rename = "openvas")]
+    Openvas,
+}
+
+impl Default for ScannerType {
+    fn default() -> Self {
+        Self::OSPD
+    }
+}
+
+impl TypedValueParser for ScannerType {
+    type Value = ScannerType;
+
+    fn parse_ref(
+        &self,
+        cmd: &clap::Command,
+        _: Option<&clap::Arg>,
+        value: &std::ffi::OsStr,
+    ) -> Result<Self::Value, clap::Error> {
+        Ok(match value.to_str().unwrap_or_default() {
+            "ospd" => ScannerType::OSPD,
+            "openvas" => ScannerType::Openvas,
+            x => {
+                let mut cmd = cmd.clone();
+                let err = cmd.error(
+                    clap::error::ErrorKind::InvalidValue,
+                    format!("`{x}` is not a scanner type."),
+                );
+                return Err(err);
+            }
+        })
+    }
+}
+
 #[derive(Deserialize, Serialize, Debug, Clone)]
 pub struct OspdWrapper {
-    pub result_check_interval: Duration,
     pub socket: PathBuf,
     pub read_timeout: Option<Duration>,
 }
@@ -29,7 +107,6 @@ pub struct OspdWrapper {
 impl Default for OspdWrapper {
     fn default() -> Self {
         OspdWrapper {
-            result_check_interval: Duration::from_secs(1),
             socket: PathBuf::from("/var/run/ospd/ospd.sock"),
             read_timeout: None,
         }
@@ -46,6 +123,23 @@ impl Default for Feed {
     }
 }
 
+impl Default for Notus {
+    fn default() -> Self {
+        Notus {
+            products_path: PathBuf::from("/var/lib/notus/products"),
+            advisories_path: PathBuf::from("/var/lib/notus/advisories"),
+        }
+    }
+}
+
+impl Default for Redis {
+    fn default() -> Self {
+        Redis {
+            url: "unix:///run/redis-openvas/redis.sock".to_string(),
+        }
+    }
+}
+
 #[derive(Deserialize, Serialize, Debug, Clone)]
 pub struct Listener {
     pub address: SocketAddr,
@@ -59,6 +153,45 @@ impl Default for Listener {
     }
 }
 
+#[derive(Deserialize, Serialize, Debug, Clone, Default, PartialEq, Eq)]
+/// Describes different modes openvasd can be run as.
+///
+/// `Service` prefix means that openvasd is a http service used by others.
+/// `Client` prefix means that openvasd is run as a client and  calls other services.
+pub enum Mode {
+    #[default]
+    #[serde(rename = "service")]
+    /// Enables all endpoints and monitors feed and active scans.
+    Service,
+    /// Disables everything but the notus endpoints
+    #[serde(rename = "service_notus")]
+    ServiceNotus,
+}
+
+impl TypedValueParser for Mode {
+    type Value = Self;
+
+    fn parse_ref(
+        &self,
+        cmd: &clap::Command,
+        _arg: Option<&clap::Arg>,
+        value: &std::ffi::OsStr,
+    ) -> Result<Self::Value, clap::Error> {
+        Ok(match value.to_str().unwrap_or_default() {
+            "service" => Self::Service,
+            "service_notus" => Self::ServiceNotus,
+            _ => {
+                let mut cmd = cmd.clone();
+                let err = cmd.error(
+                    clap::error::ErrorKind::InvalidValue,
+                    "`{}` is not a scanner type.",
+                );
+                return Err(err);
+            }
+        })
+    }
+}
+
 #[derive(Deserialize, Serialize, Debug, Clone, Default)]
 pub struct Endpoints {
     pub enable_get_scans: bool,
@@ -94,6 +227,8 @@ pub enum StorageType {
     InMemory,
     #[serde(rename = "fs")]
     FileSystem,
+    #[serde(rename = "redis")]
+    Redis,
 }
 
 impl TypedValueParser for StorageType {
@@ -108,11 +243,12 @@ impl TypedValueParser for StorageType {
         Ok(match value.to_str().unwrap_or_default() {
             "fs" => StorageType::FileSystem,
             "inmemory" => StorageType::InMemory,
-            _ => {
+            "redis" => StorageType::Redis,
+            x => {
                 let mut cmd = cmd.clone();
                 let err = cmd.error(
                     clap::error::ErrorKind::InvalidValue,
-                    "`{}` is not an storage type.",
+                    format!("`{x}` is not a storage type."),
                 );
                 return Err(err);
             }
@@ -141,24 +277,32 @@ pub struct Storage {
     pub storage_type: StorageType,
     #[serde(default)]
     pub fs: FileStorage,
+    #[serde(default)]
+    pub redis: Redis,
 }
 
 #[derive(Deserialize, Serialize, Debug, Clone, Default)]
 pub struct Config {
+    #[serde(default)]
+    pub mode: Mode,
     #[serde(default)]
     pub feed: Feed,
     #[serde(default)]
+    pub notus: Notus,
+    #[serde(default)]
     pub endpoints: Endpoints,
     #[serde(default)]
     pub tls: Tls,
     #[serde(default)]
-    pub ospd: OspdWrapper,
+    pub scanner: Scanner,
     #[serde(default)]
     pub listener: Listener,
     #[serde(default)]
     pub log: Logging,
     #[serde(default)]
     pub storage: Storage,
+    #[serde(default)]
+    pub scheduler: Scheduler,
 }
 
 impl Display for Config {
@@ -204,7 +348,7 @@ impl Config {
             )
             .arg(
                 clap::Arg::new("feed-path")
-                    .env("FEEED_PATH")
+                    .env("FEED_PATH")
                     .long("feed-path")
                     .value_parser(clap::builder::PathBufValueParser::new())
                     .action(ArgAction::Set)
@@ -226,6 +370,27 @@ impl Config {
                     .value_name("SECONDS")
                     .help("interval to check for feed updates in seconds"),
             )
+            .arg(
+                clap::Arg::new("notus-advisories")
+                    .env("NOTUS_ADVISORIES")
+                    .long("advisories")
+                    .value_parser(clap::builder::PathBufValueParser::new())
+                    .action(ArgAction::Set)
+                    .help("Path containing the Notus advisories directory"))
+            .arg(
+                clap::Arg::new("notus-products")
+                    .env("NOTUS_PRODUCTS")
+                    .long("products")
+                    .value_parser(clap::builder::PathBufValueParser::new())
+                    .action(ArgAction::Set)
+                    .help("Path containing the Notus products directory"))
+            .arg(
+                clap::Arg::new("redis-url")
+                    .long("redis-url")
+                    .env("REDIS_URL")
+                    //.value_parser(clap::builder::PathBufValueParser::new())
+                    .action(ArgAction::Set)
+                    .help("Redis url. Either unix:// or redis://"))
             .arg(
                 clap::Arg::new("tls-certs")
                     .env("TLS_CERTS")
@@ -264,6 +429,44 @@ impl Config {
                     .action(ArgAction::Set)
                     .help("API key that must be set as X-API-KEY header to gain access"),
             )
+            .arg(
+                clap::Arg::new("scanner-type")
+                    .env("SCANNER_TYPE")
+                    .long("scanner-type")
+                    .value_name("ospd,openvas")
+                    .value_parser(ScannerType::OSPD)
+                    .help("Type of scanner used to manage scans")
+            )
+            .arg(
+                clap::Arg::new("max-queued-scans")
+                    .env("MAX_QUEUED_SCANS")
+                    .long("max-queued-scans")
+                    .action(ArgAction::Set)
+                    .help("Maximum number of queued scans")
+            )
+            .arg(
+                clap::Arg::new("max-running-scans")
+                    .env("MAX_RUNNING_SCANS")
+                    .long("max-running-scans")
+                    .action(ArgAction::Set)
+                    .help("Maximum number of active running scans, omit for no limits")
+
+            )
+            .arg(
+                clap::Arg::new("min-free-mem")
+                    .env("MIN_FREE_MEMORY")
+                    .long("min-free-mem")
+                    .action(ArgAction::Set)
+                    .help("Minimum memory available to start a new scan")
+            )
+            .arg(
+                clap::Arg::new("check-interval")
+                    .env("SCHEDULER_CHECK_INTERVAL")
+                    .long("check_interval")
+                    .value_parser(clap::value_parser!(u64))
+                    .value_name("SECONDS")
+                    .help("Check interval of the Scheduler if a new scan can be started")
+            )
             .arg(
                 clap::Arg::new("ospd-socket")
                     .env("OSPD_SOCKET")
@@ -300,8 +503,8 @@ impl Config {
                 clap::Arg::new("storage_type")
                     .env("STORAGE_TYPE")
                     .long("storage-type")
-                    .value_name("fs,inmemory")
                     .value_parser(StorageType::InMemory)
+                    .value_name("redis,inmemory,fs")
                     .help("either be stored in memory or on the filesystem."),
             )
             .arg(
@@ -326,6 +529,14 @@ impl Config {
                     .short('L')
                     .help("Level of log messages to be shown. TRACE > DEBUG > INFO > WARN > ERROR"),
             )
+            .arg(
+                clap::Arg::new("mode")
+                    .env("OPENVASD_MODE")
+                    .long("mode")
+                    .value_name("service,service_notus")
+                    .value_parser(Mode::Service)
+                    .help("Sets the openvasd mode"),
+            )
             .get_matches();
         let mut config = match cmds.get_one::<String>("config") {
             Some(path) => Self::from_file(path),
@@ -340,18 +551,39 @@ impl Config {
         if let Some(interval) = cmds.get_one::<u64>("feed-check-interval") {
             config.feed.check_interval = Duration::from_secs(*interval);
         }
-        if let Some(interval) = cmds.get_one::<u64>("result-check-interval") {
-            config.ospd.result_check_interval = Duration::from_secs(*interval);
+        if let Some(scanner_type) = cmds.get_one::<ScannerType>("scanner-type") {
+            config.scanner.scanner_type = scanner_type.clone()
+        }
+        if let Some(max_queued_scans) = cmds.get_one::<usize>("max-queued-scans") {
+            config.scheduler.max_queued_scans = Some(*max_queued_scans)
+        }
+        if let Some(max_running_scans) = cmds.get_one::<usize>("max-running-scans") {
+            config.scheduler.max_running_scans = Some(*max_running_scans)
+        }
+        if let Some(min_free_mem) = cmds.get_one::<u64>("min-free-mem") {
+            config.scheduler.min_free_mem = Some(*min_free_mem)
+        }
+        if let Some(check_interval) = cmds.get_one::<u64>("check-interval") {
+            config.scheduler.check_interval = Duration::from_millis(*check_interval)
         }
         if let Some(path) = cmds.get_one::<PathBuf>("ospd-socket") {
-            config.ospd.socket = path.clone();
+            config.scanner.ospd.socket.clone_from(path);
         }
         if let Some(interval) = cmds.get_one::<u64>("read-timeout") {
-            config.ospd.read_timeout = Some(Duration::from_secs(*interval));
+            config.scanner.ospd.read_timeout = Some(Duration::from_secs(*interval));
         }
 
         if let Some(path) = cmds.get_one::<PathBuf>("feed-path") {
-            config.feed.path = path.clone();
+            config.feed.path.clone_from(path);
+        }
+        if let Some(path) = cmds.get_one::<PathBuf>("notus-products") {
+            config.notus.products_path.clone_from(path);
+        }
+        if let Some(path) = cmds.get_one::<PathBuf>("notus-advisories") {
+            config.notus.advisories_path.clone_from(path);
+        }
+        if let Some(path) = cmds.get_one::<String>("redis-url") {
+            config.storage.redis.url.clone_from(path);
         }
         if let Some(path) = cmds.get_one::<PathBuf>("tls-certs") {
             config.tls.certs = Some(path.clone());
@@ -372,13 +604,16 @@ impl Config {
             config.listener.address = *ip;
         }
         if let Some(log_level) = cmds.get_one::<String>("log-level") {
-            config.log.level = log_level.clone();
+            config.log.level.clone_from(log_level);
         }
         if let Some(stype) = cmds.get_one::<StorageType>("storage_type") {
             config.storage.storage_type = stype.clone();
         }
         if let Some(path) = cmds.get_one::<PathBuf>("storage_path") {
-            config.storage.fs.path = path.clone();
+            config.storage.fs.path.clone_from(path);
+        }
+        if let Some(mode) = cmds.get_one::<Mode>("mode") {
+            config.mode = mode.clone();
         }
         if let Some(key) = cmds.get_one::<String>("storage_key") {
             if !key.is_empty() {
@@ -412,13 +647,24 @@ mod tests {
         assert!(config.tls.key.is_none());
         assert!(config.tls.client_certs.is_none());
 
-        assert_eq!(config.ospd.result_check_interval, Duration::from_secs(1));
-        assert_eq!(config.ospd.socket, PathBuf::from("/var/run/ospd/ospd.sock"));
-        assert!(config.ospd.read_timeout.is_none());
+        assert_eq!(config.scheduler.check_interval, Duration::from_millis(500));
+        assert_eq!(
+            config.scanner.ospd.socket,
+            PathBuf::from("/var/run/ospd/ospd.sock")
+        );
+        assert!(config.scanner.ospd.read_timeout.is_none());
 
         assert_eq!(config.listener.address, ([127, 0, 0, 1], 3000).into());
 
         assert_eq!(config.log.level, "INFO".to_string());
+        // this is used to verify the default config manually.
+        // se to true to write the default configuration to `tmp`
+        if false {
+            let mut cf = std::fs::File::create("/tmp/openvas.default.example.toml").unwrap();
+            use std::io::Write;
+            cf.write_all(toml::to_string_pretty(&config).unwrap().as_bytes())
+                .unwrap();
+        }
     }
 
     #[test]
