@@ -1,15 +1,15 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
-
-use std::{path::PathBuf, sync::RwLock};
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
 use async_trait::async_trait;
+use std::sync::{Arc, RwLock};
 use storage::DefaultDispatcher;
 
-use crate::{
-    response,
-    scan::{Error, ScanDeleter, ScanResultFetcher, ScanStarter, ScanStopper},
+use crate::{config, notus::NotusWrapper, response, scheduling, tls::TlsConfig};
+
+use models::scanner::{
+    Error, ScanDeleter, ScanResultFetcher, ScanResults, ScanStarter, ScanStopper,
 };
 
 #[derive(Debug, Clone)]
@@ -17,56 +17,20 @@ pub struct NoScanner;
 #[derive(Debug, Clone)]
 pub struct Scanner<S>(S);
 
-#[derive(Debug, Clone)]
-/// sets the interval when to check for new results.
-pub struct ResultContext(pub std::time::Duration);
-
-impl From<std::time::Duration> for ResultContext {
-    fn from(d: std::time::Duration) -> Self {
-        Self(d)
-    }
-}
-
-#[derive(Debug, Clone)]
-/// Sets the path to the feed and the interval when to check for updates.
-pub struct FeedContext {
-    /// The path to the feed.
-    pub path: PathBuf,
-    /// The interval when to check for updates.
-    pub verify_interval: std::time::Duration,
-    /// Enable signature check
-    pub signature_check: bool,
-}
-
-impl From<(PathBuf, std::time::Duration, bool)> for FeedContext {
-    fn from(
-        (path, verify_interval, signature_check): (PathBuf, std::time::Duration, bool),
-    ) -> Self {
-        Self {
-            path,
-            verify_interval,
-            signature_check,
-        }
-    }
-}
-
-impl From<(&str, std::time::Duration, bool)> for FeedContext {
-    fn from((path, verify_interval, signature_check): (&str, std::time::Duration, bool)) -> Self {
-        (PathBuf::from(path), verify_interval, signature_check).into()
-    }
-}
-
-#[derive(Debug, Clone, Default)]
+#[derive(Debug, Default)]
 /// Context builder is used to build the context of the application.
 pub struct ContextBuilder<S, DB, T> {
     scanner: T,
     storage: DB,
-    result_config: Option<ResultContext>,
-    feed_config: Option<FeedContext>,
+    feed_config: Option<crate::config::Feed>,
     api_key: Option<String>,
+    tls_config: Option<TlsConfig>,
     enable_get_scans: bool,
     marker: std::marker::PhantomData<S>,
     response: response::Response,
+    notus: Option<NotusWrapper>,
+    scheduler_config: Option<config::Scheduler>,
+    mode: config::Mode,
 }
 
 impl<S>
@@ -77,29 +41,31 @@ impl<S>
         Self {
             scanner: NoScanner,
             storage: crate::storage::inmemory::Storage::default(),
-            result_config: None,
             feed_config: None,
             api_key: None,
+            tls_config: None,
             marker: std::marker::PhantomData,
             enable_get_scans: false,
             response: response::Response::default(),
+            notus: None,
+            scheduler_config: None,
+            mode: config::Mode::default(),
         }
     }
 }
 
 impl<S, DB, T> ContextBuilder<S, DB, T> {
-    /// Sets the result config.
-    pub fn result_config(mut self, config: impl Into<ResultContext>) -> Self {
-        self.result_config = Some(config.into());
+    /// Sets the mode.
+    pub fn mode(mut self, mode: config::Mode) -> Self {
+        self.mode = mode;
         self
     }
-
     /// Sets the feed config.
-    pub fn feed_config(mut self, config: impl Into<FeedContext>) -> Self {
-        self.feed_config = Some(config.into());
+    pub fn feed_config(mut self, config: config::Feed) -> Self {
+        self.feed_config = Some(config);
         if let Some(fp) = self.feed_config.as_ref() {
             let loader = nasl_interpreter::FSPluginLoader::new(fp.path.clone());
-            let dispatcher: DefaultDispatcher<String> = DefaultDispatcher::default();
+            let dispatcher: DefaultDispatcher = DefaultDispatcher::default();
             let version =
                 feed::version(&loader, &dispatcher).unwrap_or_else(|_| String::from("UNDEFINED"));
             self.response.set_feed_version(&version);
@@ -110,9 +76,12 @@ impl<S, DB, T> ContextBuilder<S, DB, T> {
     /// Sets the api key.
     pub fn api_key(mut self, api_key: impl Into<Option<String>>) -> Self {
         self.api_key = api_key.into();
-        if self.api_key.is_some() {
-            self.response.add_authentication("x-api-key");
-        }
+        self
+    }
+
+    /// Set the TLS config
+    pub fn tls_config(mut self, tls_config: Option<TlsConfig>) -> Self {
+        self.tls_config = tls_config;
         self
     }
 
@@ -122,75 +91,131 @@ impl<S, DB, T> ContextBuilder<S, DB, T> {
         self
     }
 
+    /// Set notus
+    pub fn notus(mut self, notus: NotusWrapper) -> Self {
+        self.notus = Some(notus);
+        self
+    }
+
+    pub fn scheduler_config(mut self, scheduler_config: config::Scheduler) -> Self {
+        self.scheduler_config = Some(scheduler_config);
+        self
+    }
+
     /// Sets the storage.
     #[allow(dead_code)]
     pub fn storage<NDB>(self, storage: NDB) -> ContextBuilder<S, NDB, T> {
         let ContextBuilder {
             scanner,
             storage: _,
-            result_config,
             feed_config,
             api_key,
+            tls_config,
             enable_get_scans,
             marker,
             response,
+            notus,
+            scheduler_config,
+            mode,
         } = self;
         ContextBuilder {
             scanner,
             storage,
-            result_config,
             feed_config,
             api_key,
+            tls_config,
             enable_get_scans,
             marker,
             response,
+            notus,
+            scheduler_config,
+            mode,
         }
     }
 }
 
-impl<S, DB> ContextBuilder<S, DB, NoScanner>
-where
-    S: Clone,
-{
+impl<S, DB> ContextBuilder<S, DB, NoScanner> {
     /// Sets the scanner. This is required.
     pub fn scanner(self, scanner: S) -> ContextBuilder<S, DB, Scanner<S>>
     where
-        S: super::Scanner + 'static + std::marker::Send + std::marker::Sync + std::fmt::Debug,
+        S: models::scanner::Scanner
+            + 'static
+            + std::marker::Send
+            + std::marker::Sync
+            + std::fmt::Debug,
     {
         let Self {
-            result_config,
             feed_config,
             api_key,
+            tls_config,
             enable_get_scans,
             scanner: _,
             marker: _,
             response,
             storage,
+            notus,
+            scheduler_config,
+            mode,
         } = self;
         ContextBuilder {
             scanner: Scanner(scanner),
             storage,
-            result_config,
             feed_config,
             marker: std::marker::PhantomData,
             api_key,
+            tls_config,
             enable_get_scans,
             response,
+            notus,
+            scheduler_config,
+            mode,
         }
     }
 }
 
 impl<S, DB> ContextBuilder<S, DB, Scanner<S>> {
-    pub fn build(self) -> Context<S, DB> {
+    fn configure_authentication_methods(&mut self) {
+        let tls_config = if let Some(tls_config) = self.tls_config.take() {
+            if tls_config.has_clients && self.api_key.is_some() {
+                tracing::warn!("Client certificates and api key are configured. To disable the possibility to bypass client verification the API key is ignored.");
+                self.api_key = None;
+            }
+            Some(tls_config)
+        } else {
+            None
+        };
+        self.tls_config = tls_config;
+        match (self.tls_config.is_some(), self.api_key.is_some()) {
+            (true, true) => unreachable!(),
+            (true, false) => self.response.add_authentication("mTLS"),
+            (false, true) => self.response.add_authentication("x-api-key"),
+            (false, false) => {
+                tracing::warn!(
+                    "Neither mTLS nor an API key are set. /scans endpoint is unsecured."
+                );
+            }
+        }
+    }
+
+    pub fn build(mut self) -> Context<S, DB> {
+        self.configure_authentication_methods();
+        let scheduler = scheduling::Scheduler::new(
+            self.scheduler_config.unwrap_or_default(),
+            self.scanner.0,
+            self.storage,
+        );
+        let shared_feed = Arc::clone(&scheduler.feed_version());
+        self.response.add_feed_version(shared_feed);
         Context {
-            scanner: self.scanner.0,
             response: self.response,
-            db: self.storage,
-            result_config: self.result_config,
+            scheduler,
             feed_config: self.feed_config,
             abort: Default::default(),
             api_key: self.api_key,
+            tls_config: self.tls_config,
             enable_get_scans: self.enable_get_scans,
+            notus: self.notus,
+            mode: self.mode,
         }
     }
 }
@@ -198,27 +223,27 @@ impl<S, DB> ContextBuilder<S, DB, Scanner<S>> {
 #[derive(Debug)]
 /// The context of the application
 pub struct Context<S, DB> {
-    /// The scanner that is used to start, stop and fetch results of scans.
-    pub scanner: S,
     /// Creates responses
     pub response: response::Response,
-    /// The scans that are being tracked.
-    ///
-    /// It is locked to allow concurrent access, usually the results are updated
-    /// with a background task and appended to the progress of the scan.
-    pub db: DB,
-    /// Configuration for result fetching
-    pub result_config: Option<ResultContext>,
     /// Configuration for feed handling.
-    pub feed_config: Option<FeedContext>,
+    pub feed_config: Option<config::Feed>,
     /// The api key that is used to authenticate the client.
     ///
     /// When none api key is set, no authentication is required.
     pub api_key: Option<String>,
+    pub tls_config: Option<TlsConfig>,
     /// Whether to enable the GET /scans endpoint
     pub enable_get_scans: bool,
+    pub mode: config::Mode,
     /// Aborts the background loops
     pub abort: RwLock<bool>,
+    /// Notus Scanner
+    pub notus: Option<NotusWrapper>,
+    /// All scanner and db operations must go through a scheduler.
+    ///
+    /// This allows us to throttle requests per need and gives us control when to start/stop/delete
+    /// a scan.
+    pub scheduler: scheduling::Scheduler<DB, S>,
 }
 
 #[derive(Debug, Clone, Default)]
@@ -230,6 +255,10 @@ impl ScanStarter for NoOpScanner {
     async fn start_scan(&self, _: models::Scan) -> Result<(), Error> {
         Ok(())
     }
+
+    async fn can_start_scan(&self, _: &models::Scan) -> bool {
+        true
+    }
 }
 
 #[async_trait]
@@ -254,7 +283,7 @@ impl ScanDeleter for NoOpScanner {
 
 #[async_trait]
 impl ScanResultFetcher for NoOpScanner {
-    async fn fetch_results<I>(&self, _: I) -> Result<crate::scan::FetchResult, Error>
+    async fn fetch_results<I>(&self, _: I) -> Result<ScanResults, Error>
     where
         I: AsRef<str> + Send,
     {
