@@ -1,20 +1,30 @@
 // SPDX-FileCopyrightText: 2023 Greenbone AG
 //
-// SPDX-License-Identifier: GPL-2.0-or-later
+// SPDX-License-Identifier: GPL-2.0-or-later WITH x11vnc-openssl-exception
 
-use std::marker::PhantomData;
+use std::collections::BTreeMap;
+use std::fmt::Debug;
+
+use std::str::FromStr;
 use std::sync::Arc;
 use std::sync::Mutex;
+use std::sync::MutexGuard;
 
 use crate::dberror::DbError;
 use crate::dberror::RedisStorageResult;
+use itertools::Itertools;
 use redis::*;
 
-use storage::nvt::Nvt;
-use storage::nvt::NvtPreference;
-use storage::nvt::NvtRef;
-use storage::nvt::PerNVTDispatcher;
+use storage::item::Nvt;
+use storage::item::NvtPreference;
+use storage::item::NvtRef;
+use storage::item::PerItemDispatcher;
+use storage::item::TagKey;
+use storage::item::TagValue;
+use storage::ContextKey;
 use storage::Kb;
+use storage::NotusAdvisory;
+
 use storage::StorageError;
 
 enum KbNvtPos {
@@ -25,26 +35,30 @@ enum KbNvtPos {
     RequiredUDPPorts,
     RequiredPorts,
     Dependencies,
+    Tags,
+    Cves,
+    Bids,
+    Xrefs,
     Category,
     Family,
     Name,
 }
 
-impl TryFrom<storage::nvt::NVTKey> for KbNvtPos {
+impl TryFrom<storage::item::NVTKey> for KbNvtPos {
     type Error = StorageError;
 
-    fn try_from(value: storage::nvt::NVTKey) -> Result<Self, Self::Error> {
+    fn try_from(value: storage::item::NVTKey) -> Result<Self, Self::Error> {
         Ok(match value {
-            storage::nvt::NVTKey::FileName => Self::Filename,
-            storage::nvt::NVTKey::Name => Self::Name,
-            storage::nvt::NVTKey::Dependencies => Self::Dependencies,
-            storage::nvt::NVTKey::RequiredKeys => Self::RequiredKeys,
-            storage::nvt::NVTKey::MandatoryKeys => Self::MandatoryKeys,
-            storage::nvt::NVTKey::ExcludedKeys => Self::ExcludedKeys,
-            storage::nvt::NVTKey::RequiredPorts => Self::RequiredPorts,
-            storage::nvt::NVTKey::RequiredUdpPorts => Self::RequiredUDPPorts,
-            storage::nvt::NVTKey::Category => Self::Category,
-            storage::nvt::NVTKey::Family => Self::Family,
+            storage::item::NVTKey::FileName => Self::Filename,
+            storage::item::NVTKey::Name => Self::Name,
+            storage::item::NVTKey::Dependencies => Self::Dependencies,
+            storage::item::NVTKey::RequiredKeys => Self::RequiredKeys,
+            storage::item::NVTKey::MandatoryKeys => Self::MandatoryKeys,
+            storage::item::NVTKey::ExcludedKeys => Self::ExcludedKeys,
+            storage::item::NVTKey::RequiredPorts => Self::RequiredPorts,
+            storage::item::NVTKey::RequiredUdpPorts => Self::RequiredUDPPorts,
+            storage::item::NVTKey::Category => Self::Category,
+            storage::item::NVTKey::Family => Self::Family,
             // tags must also be handled manually due to differentiation
             _ => {
                 return Err(StorageError::UnexpectedData(format!(
@@ -54,10 +68,34 @@ impl TryFrom<storage::nvt::NVTKey> for KbNvtPos {
         })
     }
 }
-
+#[derive(Default)]
 pub struct RedisCtx {
-    kb: Connection, //a redis connection
-    pub db: u32,    // the name space
+    kb: Option<Connection>, //a redis connection
+    pub db: u32,            // the name space
+}
+
+impl Debug for RedisCtx {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "Redis connection. Db {}", self.db)
+    }
+}
+
+#[derive(Debug, PartialEq, Eq)]
+struct RedisVectorHandler {
+    v: Vec<String>,
+}
+
+impl FromRedisValue for RedisVectorHandler {
+    fn from_redis_value(v: &Value) -> redis::RedisResult<RedisVectorHandler> {
+        match v {
+            Value::Nil => Ok(RedisVectorHandler { v: Vec::new() }),
+            _ => {
+                let new_var: String = from_redis_value(v).unwrap_or_default();
+                let nv = vec![new_var];
+                Ok(RedisVectorHandler { v: nv })
+            }
+        }
+    }
 }
 
 #[derive(Debug, PartialEq, Eq)]
@@ -89,6 +127,7 @@ pub enum NameSpaceSelector {
 }
 
 const CACHE_KEY: &str = "nvticache";
+const NOTUS_KEY: &str = "notuscache";
 const DB_INDEX: &str = "GVM.__GlobalDBIndex";
 
 impl NameSpaceSelector {
@@ -149,26 +188,269 @@ impl NameSpaceSelector {
 /// Default selector for a feed-update run
 pub const FEEDUPDATE_SELECTOR: &[NameSpaceSelector] =
     &[NameSpaceSelector::Key(CACHE_KEY), NameSpaceSelector::Free];
+pub const NOTUSUPDATE_SELECTOR: &[NameSpaceSelector] =
+    &[NameSpaceSelector::Key(NOTUS_KEY), NameSpaceSelector::Free];
 
 pub trait RedisWrapper {
     fn rpush<T: ToRedisArgs>(&mut self, key: &str, val: T) -> RedisStorageResult<()>;
     fn lpush<T: ToRedisArgs>(&mut self, key: &str, val: T) -> RedisStorageResult<()>;
+    fn del(&mut self, key: &str) -> RedisStorageResult<()>;
+    fn lindex(&mut self, key: &str, index: isize) -> RedisStorageResult<String>;
+    fn lrange(&mut self, key: &str, start: isize, end: isize) -> RedisStorageResult<Vec<String>>;
+    fn keys(&mut self, pattern: &str) -> RedisStorageResult<Vec<String>>;
+    fn pop(&mut self, pattern: &str) -> RedisStorageResult<Vec<String>>;
 }
 
 impl RedisWrapper for RedisCtx {
     ///Wrapper function to avoid accessing kb member directly.
     #[inline(always)]
     fn rpush<T: ToRedisArgs>(&mut self, key: &str, val: T) -> RedisStorageResult<()> {
-        self.kb.rpush(key, val).map_err(|e| e.into())
+        self.kb
+            .as_mut()
+            .expect("Valid redis connection")
+            .rpush(key, val)
+            .map_err(|e| e.into())
     }
 
     ///Wrapper function to avoid accessing kb member directly.
     #[inline(always)]
     fn lpush<T: ToRedisArgs>(&mut self, key: &str, val: T) -> RedisStorageResult<()> {
-        self.kb.lpush(key, val).map_err(|e| e.into())
+        self.kb
+            .as_mut()
+            .expect("Valid redis connection")
+            .lpush(key, val)
+            .map_err(|e| e.into())
+    }
+
+    ///Wrapper function to avoid accessing kb member directly.
+    #[inline(always)]
+    fn del(&mut self, key: &str) -> RedisStorageResult<()> {
+        self.kb
+            .as_mut()
+            .expect("Valid redis connection")
+            .del(key)
+            .map_err(|e| e.into())
+    }
+
+    ///Wrapper function to avoid accessing kb member directly.
+    #[inline(always)]
+    fn lindex(&mut self, key: &str, index: isize) -> RedisStorageResult<String> {
+        let ret: RedisValueHandler = self
+            .kb
+            .as_mut()
+            .expect("Valid redis connection")
+            .lindex(key, index)?;
+        Ok(ret.v)
+    }
+
+    ///Wrapper function to avoid accessing kb member directly.
+    #[inline(always)]
+    fn lrange(&mut self, key: &str, start: isize, end: isize) -> RedisStorageResult<Vec<String>> {
+        let ret = self
+            .kb
+            .as_mut()
+            .expect("Valid redis connection")
+            .lrange(key, start, end)?;
+        Ok(ret)
+    }
+
+    ///Wrapper function to avoid accessing kb member directly.
+    #[inline(always)]
+    fn keys(&mut self, pattern: &str) -> RedisStorageResult<Vec<String>> {
+        let ret: Vec<String> = self
+            .kb
+            .as_mut()
+            .expect("Valid redis connection")
+            .keys(pattern)?;
+        Ok(ret)
+    }
+
+    fn pop(&mut self, key: &str) -> RedisStorageResult<Vec<String>> {
+        let ret: (Vec<String>,) = redis::pipe()
+            .cmd("LRANGE")
+            .arg(key)
+            .arg("0")
+            .arg("-1")
+            .cmd("DEL")
+            .arg(key)
+            .ignore()
+            .query(&mut self.kb.as_mut().unwrap())
+            .unwrap();
+        // Since items are lpushed, the returned vector must be reversed to keep the order.
+        let mut status = ret.0;
+        status.reverse();
+
+        Ok(status)
     }
 }
 
+pub trait RedisAddAdvisory: RedisWrapper {
+    /// Add an NVT in the redis cache.
+    ///
+    /// The NVT metadata is stored in two different keys:
+
+    /// - 'nvt:<OID>': stores the general metadata ordered following the KbNvtPos indexes
+    /// - 'oid:<OID>:prefs': stores the plugins preferences, including the script_timeout
+    ///   (which is especial and uses preferences id 0)
+    fn redis_add_advisory(
+        &mut self,
+        _: &str,
+        adv: Option<NotusAdvisory>,
+    ) -> RedisStorageResult<()> {
+        match adv {
+            Some(data) => {
+                let key = format!("internal/notus/advisories/{}", &data.adv.oid);
+                let value = models::Vulnerability::from(data);
+                let value = serde_json::to_string(&value)
+                    .map_err(|e| DbError::Unknown(format!("Serialization error: {e}")))?;
+                self.rpush(&key, value)?;
+            }
+            None => self.rpush(NOTUS_KEY, "1".to_string())?,
+        };
+        Ok(())
+    }
+}
+
+impl RedisAddAdvisory for RedisCtx {}
+
+pub trait RedisGetNvt: RedisWrapper {
+    #[inline(always)]
+    fn get_refs(bids: &str, cves: &str, xrefs: &str) -> Vec<NvtRef> {
+        let f = |x: &str| match x.split_once(':') {
+            Some((a, b)) => NvtRef::from((a, b)),
+            None => NvtRef::from(("", "")),
+        };
+        let mut bid_refs: Vec<NvtRef> =
+            bids.split(", ").map(|r| NvtRef::from(("bid", r))).collect();
+        let mut cve_refs: Vec<NvtRef> =
+            cves.split(", ").map(|r| NvtRef::from(("cve", r))).collect();
+        let mut xrefs_refs = xrefs.split(", ").map(f).collect();
+
+        let mut refs: Vec<NvtRef> = Vec::new();
+        refs.append(&mut bid_refs);
+        refs.append(&mut cve_refs);
+        refs.append(&mut xrefs_refs);
+        refs
+    }
+
+    #[inline(always)]
+    fn get_prefs(&mut self, oid: &str) -> RedisStorageResult<Vec<NvtPreference>> {
+        let keyname = format!("oid:{}:prefs", oid);
+        let mut prefs_list = self.lrange(&keyname, 0, -1)?;
+        let mut prefs: Vec<NvtPreference> = Vec::new();
+        for p in prefs_list.iter_mut() {
+            if let Some(sp) = p
+                .splitn(4, "|||")
+                .collect_tuple::<(&str, &str, &str, &str)>()
+            {
+                prefs.push(NvtPreference::from(sp));
+            }
+        }
+        Ok(prefs)
+    }
+
+    #[inline(always)]
+    fn get_tags(tags: &str) -> BTreeMap<TagKey, TagValue> {
+        let mut tag_map = BTreeMap::new();
+
+        let tag_list = tags.split('|').map(|x| {
+            x.splitn(2, '=')
+                .collect_tuple::<(&str, &str)>()
+                .unwrap_or_default()
+        });
+
+        for (k, v) in tag_list.into_iter() {
+            if let Ok(tk) = TagKey::from_str(k) {
+                match tk {
+                    TagKey::CreationDate | TagKey::LastModification | TagKey::SeverityDate => {
+                        tag_map.insert(
+                            tk,
+                            TagValue::from(i64::from_str(v).expect("Valid timestamp")),
+                        )
+                    }
+                    _ => tag_map.insert(tk, TagValue::from(v)),
+                };
+            }
+        }
+
+        tag_map
+    }
+
+    fn redis_get_advisory(&mut self, oid: &str) -> RedisStorageResult<Option<Nvt>> {
+        let keyname = format!("internal/notus/advisories/{}", oid);
+        let nvt_data = self.lindex(&keyname, 0)?;
+        if nvt_data.is_empty() {
+            return Ok(None);
+        }
+
+        if let Ok(adv) = serde_json::from_str::<models::Vulnerability>(&nvt_data) {
+            Ok(Some(Nvt::from((oid, adv))))
+        } else {
+            Ok(None)
+        }
+    }
+    /// Nvt metadata is stored under two different keys
+    /// - 'nvt:<OID>': stores the general metadata ordered following the KbNvtPos indexes
+    /// - 'oid:<OID>:prefs': stores the plugins preferences, including the script_timeout
+    ///   (which is especial and uses preferences id 0)
+    fn redis_get_vt(&mut self, oid: &str) -> RedisStorageResult<Option<Nvt>> {
+        let keyname = format!("nvt:{}", oid);
+        let nvt_data = self.lrange(&keyname, 0, -1)?;
+
+        if nvt_data.is_empty() {
+            return Ok(None);
+        }
+
+        let nvt = Nvt {
+            oid: oid.to_string(),
+            name: nvt_data[KbNvtPos::Name as usize].clone(),
+            filename: nvt_data[KbNvtPos::Filename as usize].clone(),
+            tag: Self::get_tags(&nvt_data[KbNvtPos::Tags as usize].clone()),
+            dependencies: nvt_data[KbNvtPos::Dependencies as usize]
+                .split(',')
+                .map(|x| x.to_string())
+                .collect(),
+            required_keys: nvt_data[KbNvtPos::RequiredKeys as usize]
+                .split(',')
+                .map(|x| x.to_string())
+                .collect(),
+            mandatory_keys: nvt_data[KbNvtPos::MandatoryKeys as usize]
+                .split(',')
+                .map(|x| x.to_string())
+                .collect(),
+            excluded_keys: nvt_data[KbNvtPos::ExcludedKeys as usize]
+                .split(',')
+                .map(|x| x.to_string())
+                .collect(),
+            required_ports: nvt_data[KbNvtPos::RequiredPorts as usize]
+                .split(',')
+                .map(|x| x.to_string())
+                .collect(),
+            required_udp_ports: nvt_data[KbNvtPos::RequiredUDPPorts as usize]
+                .split(',')
+                .map(|x| x.to_string())
+                .collect(),
+            references: Self::get_refs(
+                &nvt_data[KbNvtPos::Bids as usize].clone(),
+                &nvt_data[KbNvtPos::Cves as usize].clone(),
+                &nvt_data[KbNvtPos::Xrefs as usize].clone(),
+            ),
+            preferences: Self::get_prefs(self, oid)?,
+            category: {
+                match storage::item::ACT::from_str(&nvt_data[KbNvtPos::Category as usize]) {
+                    Ok(c) => c,
+                    Err(_) => return Err(DbError::Unknown("Invalid nvt category".to_string())),
+                }
+            },
+            family: nvt_data[KbNvtPos::Family as usize].clone(),
+        };
+
+        Ok(Some(nvt))
+    }
+}
+
+impl RedisGetNvt for RedisCtx {}
+
 pub trait RedisAddNvt: RedisWrapper {
     /// Get References. It returns a tuple of three strings
     /// Each string is a references type, and each string
@@ -280,12 +562,14 @@ pub trait RedisAddNvt: RedisWrapper {
             &family,
             &name,
         ];
+        self.del(&key_name)?;
         self.rpush(&key_name, &values)?;
 
         // Add preferences
         let prefs = Self::prefs(&nvt.preferences);
         if !prefs.is_empty() {
             let key_name = format!("oid:{oid}:prefs");
+            self.del(&key_name)?;
             self.lpush(&key_name, prefs)?;
             //self.kb.lpush(&key_name, prefs)?;
         }
@@ -312,7 +596,12 @@ impl RedisCtx {
         let mut kb = client.get_connection()?;
         for s in selector {
             match s.select(&mut kb) {
-                Ok(x) => return Ok(RedisCtx { kb, db: x }),
+                Ok(x) => {
+                    return Ok(RedisCtx {
+                        kb: Some(kb),
+                        db: x,
+                    })
+                }
                 Err(DbError::NoAvailDbErr) => {}
                 Err(x) => return Err(x),
             }
@@ -322,25 +611,49 @@ impl RedisCtx {
 
     /// Delete an entry from the in-use namespace's list
     fn release_namespace(&mut self) -> RedisStorageResult<()> {
-        // Remove the entry from the hash list
-        self.kb.hdel(DB_INDEX, self.db)?;
+        // Remove the entry from the in-use list and return to the original namespace
+        redis::pipe()
+            .cmd("SELECT")
+            .arg("0")
+            .cmd("HDEL")
+            .arg(DB_INDEX)
+            .arg(self.db)
+            .cmd("SELECT")
+            .arg(self.db)
+            .ignore()
+            .query::<()>(&mut self.kb.as_mut().expect("Valid redis connection"))?;
         Ok(())
     }
 
     /// Delete all keys in the namespace and release the it
     pub fn delete_namespace(&mut self) -> RedisStorageResult<()> {
-        Cmd::new().arg("FLUSHDB").query(&mut self.kb)?;
+        Cmd::new()
+            .arg("FLUSHDB")
+            .query::<()>(&mut self.kb.as_mut().expect("Valid redis connection"))?;
         self.release_namespace()?;
         Ok(())
     }
+
+    /// Clean up the namespace.
+    pub fn flush_namespace(&mut self) -> RedisStorageResult<()> {
+        Cmd::new()
+            .arg("FLUSHDB")
+            .query::<()>(&mut self.kb.as_mut().expect("Valid redis connection"))?;
+        Ok(())
+    }
+
     //Wrapper function to avoid accessing kb member directly.
     pub fn set_value<T: ToRedisArgs>(&mut self, key: &str, val: T) -> RedisStorageResult<()> {
-        self.kb.set(key, val)?;
+        () = self
+            .kb
+            .as_mut()
+            .expect("Valid redis connection")
+            .set(key, val)?;
         Ok(())
     }
 
     pub fn value(&mut self, key: &str) -> RedisStorageResult<String> {
-        let ret: RedisValueHandler = self.kb.get(key)?;
+        let ret: RedisValueHandler = self.kb.as_mut().expect("Valid redis connection").get(key)?;
         Ok(ret.v)
     }
 }
@@ -353,19 +666,24 @@ impl RedisCtx {
 /// In this case we need to wait until we get the OID so that we can build the key additionally
 /// we need to have all references and preferences to respect the order to be downwards compatible.
 /// This should be changed when there is new OSP frontend available.
-pub struct NvtDispatcher<R, K>
+#[derive(Debug, Default)]
+pub struct CacheDispatcher<R>
 where
-    R: RedisWrapper + RedisAddNvt,
+    R: RedisWrapper + RedisAddNvt + RedisAddAdvisory + RedisGetNvt,
 {
     cache: Arc<Mutex<R>>,
     kbs: Arc<Mutex<Vec<Kb>>>,
-    phanton: PhantomData<K>,
 }
 
-impl<K> NvtDispatcher<RedisCtx, K>
-where
-    K: AsRef<str>,
-{
+impl<R: RedisWrapper + RedisAddNvt + RedisAddAdvisory + RedisGetNvt> CacheDispatcher<R> {
+    fn lock_cache(&self) -> Result<MutexGuard<'_, R>, DbError> {
+        self.cache
+            .lock()
+            .map_err(|e| DbError::PoisonedLock(format!("{e:?}")))
+    }
+}
+
+impl CacheDispatcher<RedisCtx> {
     /// Initialize and return an NVT Cache Object
     ///
     /// The redis_url must be a complete url including the used protocol e.g.:
@@ -373,13 +691,12 @@ where
     pub fn init(
         redis_url: &str,
         selector: &[NameSpaceSelector],
-    ) -> RedisStorageResult<NvtDispatcher<RedisCtx, K>> {
+    ) -> RedisStorageResult<CacheDispatcher<RedisCtx>> {
         let rctx = RedisCtx::open(redis_url, selector)?;
 
-        Ok(NvtDispatcher {
+        Ok(CacheDispatcher {
             cache: Arc::new(Mutex::new(rctx)),
             kbs: Arc::new(Mutex::new(Vec::new())),
-            phanton: PhantomData,
         })
     }
 
@@ -389,25 +706,27 @@ where
     /// before returning the underlying cache as a Dispatcher.
     pub fn as_dispatcher(
         redis_url: &str,
-    ) -> RedisStorageResult<PerNVTDispatcher<NvtDispatcher<RedisCtx, K>, K>> {
-        let cache = Self::init(redis_url, FEEDUPDATE_SELECTOR)?;
-        cache.reset()?;
-        Ok(PerNVTDispatcher::new(cache))
+        selector: &[NameSpaceSelector],
+    ) -> RedisStorageResult<PerItemDispatcher<CacheDispatcher<RedisCtx>>> {
+        let cache = Self::init(redis_url, selector)?;
+        cache.flushdb()?;
+        Ok(PerItemDispatcher::new(cache))
     }
 
     /// Reset the NVT Cache and release the redis namespace
     pub fn reset(&self) -> RedisStorageResult<()> {
-        let mut cache = Arc::as_ref(&self.cache)
-            .lock()
-            .map_err(|e| DbError::SystemError(format!("{e:?}")))?;
-        cache.delete_namespace()
+        self.lock_cache()?.delete_namespace()
+    }
+
+    /// Reset the NVT Cache. Do not release the namespace. Only ensure it is clean
+    pub fn flushdb(&self) -> RedisStorageResult<()> {
+        self.lock_cache()?.flush_namespace()
     }
 }
 
-impl<S, K> storage::nvt::NvtDispatcher<K> for NvtDispatcher<S, K>
+impl<S> storage::item::ItemDispatcher for CacheDispatcher<S>
 where
-    S: RedisWrapper + RedisAddNvt,
-    K: AsRef<str>,
+    S: RedisWrapper + RedisAddNvt + RedisAddAdvisory + RedisGetNvt,
 {
     fn dispatch_nvt(&self, nvt: Nvt) -> Result<(), StorageError> {
         let mut cache = Arc::as_ref(&self.cache).lock()?;
@@ -416,59 +735,71 @@ where
 
     fn dispatch_feed_version(&self, version: String) -> Result<(), StorageError> {
         let mut cache = Arc::as_ref(&self.cache).lock()?;
+        cache.del(CACHE_KEY)?;
         cache.rpush(CACHE_KEY, &[&version]).map_err(|e| e.into())
     }
 
-    fn dispatch_kb(&self, _: &K, kb: storage::Kb) -> Result<(), StorageError> {
+    fn dispatch_kb(&self, _: &ContextKey, kb: storage::Kb) -> Result<(), StorageError> {
         let mut kbs = self.kbs.lock().map_err(StorageError::from)?;
         kbs.push(kb);
         Ok(())
     }
+    fn dispatch_advisory(&self, key: &str, adv: Option<NotusAdvisory>) -> Result<(), StorageError> {
+        let mut cache = Arc::as_ref(&self.cache).lock()?;
+        cache.redis_add_advisory(key, adv).map_err(|e| e.into())
+    }
 }
 
-impl<S, K> storage::Retriever<K> for NvtDispatcher<S, K>
+impl<S> storage::Retriever for CacheDispatcher<S>
 where
-    S: RedisWrapper + RedisAddNvt,
+    S: RedisWrapper + RedisAddNvt + RedisAddAdvisory + RedisGetNvt,
 {
     fn retrieve(
         &self,
-        _: &K,
-        scope: &storage::Retrieve,
-    ) -> Result<Vec<storage::Field>, StorageError> {
+        _: &ContextKey,
+        scope: storage::Retrieve,
+    ) -> Result<Box<dyn Iterator<Item = storage::Field>>, StorageError> {
         Ok(match scope {
-            // currently not supported
-            storage::Retrieve::NVT(_) => Vec::new(),
-            storage::Retrieve::KB(s) => {
+            storage::Retrieve::NotusAdvisory(_)
+            | storage::Retrieve::NVT(_)
+            | storage::Retrieve::Result(_) => Box::new(Vec::new().into_iter()),
+            storage::Retrieve::KB(s) => Box::new({
                 let kbs = self.kbs.lock().map_err(StorageError::from)?;
-                kbs.iter()
-                    .filter(|x| &x.key == s)
-                    .map(|x| storage::Field::KB(x.clone()))
-                    .collect()
-            }
+                let kbs = kbs.clone();
+                kbs.into_iter()
+                    .filter(move |x| x.key == s)
+                    .map(move |x| storage::Field::KB(x.clone()))
+            }),
         })
     }
 
     fn retrieve_by_field(
         &self,
-        _: &storage::Field,
-        _: &storage::Retrieve,
-    ) -> Result<Vec<(K, Vec<storage::Field>)>, StorageError> {
-        // currently not supported
-        Ok(vec![])
+        _field: storage::Field,
+        _scope: storage::Retrieve,
+    ) -> Result<Box<dyn Iterator<Item = (ContextKey, storage::Field)>>, StorageError> {
+        todo!()
+    }
+
+    fn retrieve_by_fields(
+        &self,
+        _: Vec<storage::Field>,
+        _: storage::Retrieve,
+    ) -> storage::FieldKeyResult {
+        todo!()
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use std::marker::PhantomData;
     use std::sync::mpsc::{self, Sender, TryRecvError};
     use std::sync::{Arc, Mutex};
 
-    use storage::nvt::PerNVTDispatcher;
-    use storage::nvt::{NvtPreference, NvtRef, PreferenceType, TagKey, TagValue, ACT};
+    use storage::item::PerItemDispatcher;
+    use storage::item::{NvtPreference, NvtRef, PreferenceType, TagKey, TagValue, ACT};
     use storage::Dispatcher;
 
-    use super::{NvtDispatcher, RedisAddNvt, RedisWrapper};
+    use super::{CacheDispatcher, RedisAddAdvisory, RedisAddNvt, RedisGetNvt, RedisWrapper};
 
     #[derive(Clone)]
     struct FakeRedis {
@@ -496,10 +827,36 @@ mod tests {
                 .unwrap();
             Ok(())
         }
-    }
-    impl RedisAddNvt for FakeRedis {}
+        fn del(&mut self, _: &str) -> crate::dberror::RedisStorageResult<()> {
+            Ok(())
+        }
 
-    use storage::nvt::NVTField::*;
+        fn lindex(&mut self, _: &str, _: isize) -> crate::dberror::RedisStorageResult<String> {
+            Ok(String::new())
+        }
+
+        fn keys(&mut self, _: &str) -> crate::dberror::RedisStorageResult<Vec<String>> {
+            Ok(Vec::new())
+        }
+        fn pop(&mut self, _: &str) -> crate::dberror::RedisStorageResult<Vec<String>> {
+            Ok(Vec::new())
+        }
+
+        fn lrange(
+            &mut self,
+            _: &str,
+            _: isize,
+            _: isize,
+        ) -> crate::dberror::RedisStorageResult<Vec<String>> {
+            Ok(Vec::new())
+        }
+    }
+
+    impl RedisAddNvt for FakeRedis {}
+    impl RedisAddAdvisory for FakeRedis {}
+    impl RedisGetNvt for FakeRedis {}
+
+    use storage::item::NVTField::*;
     use storage::Field::NVT;
     #[test]
     fn transform_nvt() {
@@ -550,14 +907,12 @@ mod tests {
         let fr = FakeRedis { sender };
         let cache = Arc::new(Mutex::new(fr));
         let kbs = Arc::new(Mutex::new(Vec::new()));
-        let rcache = NvtDispatcher {
-            cache,
-            kbs,
-            phanton: PhantomData,
-        };
-        let dispatcher = PerNVTDispatcher::new(rcache);
+        let rcache = CacheDispatcher { cache, kbs };
+        let dispatcher = PerItemDispatcher::new(rcache);
         for c in commands {
-            dispatcher.dispatch(&"test.nasl", c).unwrap();
+            dispatcher
+                .dispatch(&storage::ContextKey::FileName("test.nasl".to_string()), c)
+                .unwrap();
         }
         dispatcher.on_exit().unwrap();
         let mut results = 0;
